<?php
 /**
 * About module
 * http://www.aa-team.com
 * ======================
 *
 * @package		Instaberg
 * @author		Andrei Dinca, AA-Team
 * @version		1.0
 */
! defined( 'ABSPATH' ) and exit;

if(class_exists('Instaberg_dashboard') != true) {
	class Instaberg_dashboard {
		
		const WXR_VERSION = '1.0';
		
		/**
		 * parent storage
		 *
		 * @var array
		 */
		public $parent = array();
		
		private $action = '';
		private $wp_import = null;
		
		/**
		 * The constructor
		 */
		public function __construct( $parent=array() )
		{
			// load parent
			$this->parent = $parent;
		}

		public function get_popular_cast()
		{
			$cast = array();
			$cast_results = get_terms( 'cast', array(
			    'orderby'    	=> 'count',
			    'order' 		=> 'DESC',
			    'number'		=> 12,
			    'hide_empty' 	=> 0
			) );

			if( $cast_results && count($cast_results) > 0 ){
				foreach ($cast_results as $key => $value) {

					$image_ID = get_term_meta( $value->term_id, '_thumbnail', true );

					$cast[$value->term_id] = array(
						'name' => $value->name,
						'count' => $value->count,
						'thumb' => ( trim($image_ID) != "" ? wp_get_attachment_image_src( $image_ID, array(200, 300) )[0] : '' ),
					);
				}
			}

			return $cast;
		}

		public function get_popular_products()
		{
			$products = array();
			$args = array(
				'post_type' => 'movies',
				'meta_key' => '_Instaberg_movie_hits',
				'orderby' => 'meta_value_num',
				'order' => 'DESC',
				'posts_per_page' => 12
			);
			$query = new WP_Query($args);
			
			if( (int) $query->found_posts > 0 ){
				foreach ( $query->posts as $post ) {

					$post_thumbnail_id = get_post_thumbnail_id( $post->ID );
					$poster_thumbnail = wp_get_attachment_image_src( $post_thumbnail_id, 'thumbnail' )[0];

					$products[$post->ID] = array(
						'hits' => (int) get_post_meta( $post->ID, '_Instaberg_movie_hits', true ),
						'poster_thumbnail' => $poster_thumbnail
					);
				}
			}

			return $products;
		}
		
		public function print_interface()
		{
			$html = array();
			$plugin_register = (string) get_option( "_" . Instaberg()->alias . "_register_html" );

			$html[] = '<div id="Instaberg_iw-section-content" class="Instaberg_iw-section-dashboard">';


			$html[] = 	'<div class="Instaberg_iw-section-dashboard-row">';
			
			$html[] = 		'<div class="Instaberg_iw-dashboard-box Instaberg_iw-box-size-2">';
			$html[] = 			'<div class="Instaberg_iw-dashboard-box-content">';
			$html[] = 				'<h3 class="dashboard-box-label">' . esc_html__('Product Support', 'instaberg') . '</h3>';
			$html[] = 				'<div class="Instaberg_iw-dashboard-in-box-content">';
			$html[] = 					'<div class="Instaberg_iw-row-icons">';
			$html[] = 						'<i class="fa fa-file-text-o"></i>';
			$html[] = 						'<div class="Instaberg_iw-icons-title">';
			$html[] = 							'<p class="Instaberg_iw-icons-strong"><a href="http://docs.aa-team.com/products/wzone-addon-woocommerce-ebay-affiliates/" target="_blank">' . esc_html__('Online Documentation', 'instaberg') . '</a></p>';
			$html[] = 							'<p>' . esc_html__('Please read the documentation for', 'instaberg') . '<a href="http://docs.aa-team.com/products/wzone-addon-woocommerce-ebay-affiliates/" target="_blank"> <strong>' . esc_html__('Instaberg', 'instaberg') . '</strong></p></a>';
			$html[] = 						'</div>';
			$html[] = 					'</div>';
			$html[] = 					'<div class="Instaberg_iw-row-icons">';
			$html[] = 						'<i class="fa fa-life-ring"></i>';
			$html[] = 						'<div class="Instaberg_iw-icons-title">';
			$html[] = 							'<p class="Instaberg_iw-icons-strong"' . esc_html__('>Ticket Support', 'instaberg') . '</p>';
			$html[] = 							'<p>' . esc_html__('Direct help from our qualified support team', 'instaberg') . '</p>';
			if( !$plugin_register && trim($plugin_register) == "" ){
				$html[] = 						'<a href="#" class="Instaberg_iw-dashboard-button Instaberg_iw-color-red Instaberg_iw-require_register">' . esc_html__('Unlock Now', 'instaberg') . '</a>';
			}else{
				$html[] = 						'<a href="http://support.aa-team.com/" target="_blank" class="Instaberg_iw-dashboard-button Instaberg_iw-color-yellow">' . esc_html__('Open New Ticket', 'instaberg') . '</a>';
			}
			$html[] = 						'</div>';
			$html[] = 					'</div>';
			$html[] = 					'<div class="Instaberg_iw-row-icons">';
			$html[] = 						'<i class="fa fa-money"></i>';
			$html[] = 						'<div class="Instaberg_iw-icons-title">';
			$html[] = 							'<p class="Instaberg_iw-icons-strong">' . esc_html__('Custom Modifications', 'instaberg') . '</p>';
			$html[] = 							'<p>' . esc_html__('Direct help from our qualified dev and design team', 'instaberg') . '</p>';
			if( !$plugin_register && trim($plugin_register) == "" ){
				$html[] = 						'<a href="#" class="Instaberg_iw-dashboard-button Instaberg_iw-color-red Instaberg_iw-require_register">' . esc_html__('Unlock Now', 'instaberg') . '</a>';
			}else{
				$html[] = 						'<a href="http://support.aa-team.com/custom-modifications/" target="_blank" class="Instaberg_iw-dashboard-button Instaberg_iw-color-yellow">' . esc_html__('Request a Quote', 'instaberg') . '</a>';
			}
			$html[] = 						'</div>';
			$html[] = 					'</div>';
			$html[] = 				'</div>';
			$html[] = 			'</div>';
			$html[] = 		'</div>';

			$current_user = wp_get_current_user();

			$html[] = 		'<div class="Instaberg_iw-dashboard-box Instaberg_iw-box-size-2">';
			$html[] = 			'<div class="Instaberg_iw-dashboard-box-content">';

			if( $plugin_register && trim($plugin_register) != "" ){
				$html[] = 				'<h3 class="dashboard-box-label">' . esc_html__('Your plugin is activated', 'instaberg') . '</h3>';
				$html[] = 				'<div class="Instaberg_iw-dashboard-in-box-content Instaberg_iw-aa-register-text">' . ( $plugin_register ) . '</div>';
			}else{

				$html[] = 				'<h3 class="dashboard-box-label">' . esc_html__('Plugin Activation', 'instaberg') . '</h3>';
				$html[] = 				'<div class="Instaberg_iw-dashboard-in-box-content">';
				$html[] = 					'<form class="Instaberg_iw-register_plugin" action="" method="POST">';
				$html[] = 						'<div class="Instaberg_iw-row-icons">';
				$html[] = 							'<i class="fa fa-key"></i>';
				$html[] = 							'<div class="Instaberg_iw-icons-title">';
				$html[] = 								'<p class="Instaberg_iw-icons-strong">' . esc_html__('Item Purchase Code (IPC)', 'instaberg') . '</p>';
				$html[] = 								'<p>' . esc_html__('You can learn how to find your purchase code', 'instaberg') . ' <a href="http://support.aa-team.com/themes/default/assets/images/ipc-help.png" target="_blank">' . esc_html__('here', 'instaberg') . '</a></p>';
				$html[] = 							'</div>';
				$html[] = 						'</div>';
				$html[] = 						'<input type="text" value="" style="width:100%" class="Instaberg_iw-dashboard-input" name="Instaberg_iw-validation-token" placeholder="' . esc_attr__('IPC Code', 'instaberg') . '">';
				$html[] = 						'<input type="text" value="' . ( $current_user->data->user_email ) . '" style="width:100%" class="Instaberg_iw-dashboard-input" name="Instaberg_iw-validation-email" placeholder="' . esc_attr__('Register email', 'instaberg') . '">';
				$html[] = 						'<p>' . esc_html__('Reminder! The Regular License is for One Install Only!', 'instaberg') . '</p>';
				$html[] = 						'<input type="button" class="Instaberg_iw-dashboard-button Instaberg_iw-color-yellow" value="' . esc_attr__('Register the code', 'instaberg') . '" />';
				$html[] = 					'</form>';
				$html[] = 				'</div>';
			}
			$html[] = 			'</div>';
			$html[] = 		'</div>';

			$html[] = 		'<div class="Instaberg_iw-dashboard-box Instaberg_iw-box-size-2">';
			$html[] = 			'<div class="Instaberg_iw-dashboard-box-content">';
			$html[] = 				'<h3 class="dashboard-box-label">' . esc_html__('System Requirements', 'instaberg') . '</h3>';
			$html[] = 				'<div class="Instaberg_iw-dashboard-in-box-content">';
			$html[] = 					'<table class="Instaberg_iw-system-req">';
			$html[] = 						'<tr>';
			$html[] = 							'<td width="55%">' . ( is_multisite() ? 'WPMU' : 'WP' ) . ' ' . esc_html__('Version', 'instaberg') . ':</td>';
			$html[] = 							'<td><span class="Instaberg_iw-success"><i class="fa fa-check-square"></i></span></td>';
			$html[] = 							'<td width="30%">' . get_bloginfo('version') . ' </td>';
			$html[] = 						'</tr>';

			$html[] = 						'<tr>';
			$html[] = 							'<td width="55%">' . esc_html__('Uploads folder writable', 'instaberg') . '</td>';
			$html[] = 							'<td><span class="Instaberg_iw-success"><i class="fa fa-check-square"></i></span></td>';
			$html[] = 							'<td width="30%"></td>';
			$html[] = 						'</tr>';

			$html[] = 						'<tr>';
			$html[] = 							'<td width="55%">' . esc_html__('Max Upload Size', 'instaberg') . ':</td>';
			$html[] = 							'<td><span class="Instaberg_iw-success"><i class="fa fa-exclamation-circle"></i></span></td>';
			$html[] = 							'<td width="30%">' . ( size_format( wp_max_upload_size() ) ) . '</td>';
			$html[] = 						'</tr>';


			$allow_remote_get = (boolean) get_option( "_Instaberg_allow_remote_get" );

			if( $allow_remote_get === false ){
				$response = wp_remote_get( 'http://google.com' );
				if( isset($response['body']) && trim($response['body']) != "" ){
					$allow_remote_get = true;
					update_option( "_Instaberg_allow_remote_get", true );
				}
			}

			$html[] = 						'<tr>';
			$html[] = 							'<td width="55%">' . esc_html__('Remote GET', 'instaberg') . ':</td>';
			if( $allow_remote_get ){
				$html[] = 						'<td><span class="Instaberg_iw-success"><i class="fa fa-exclamation-circle"></i></span></td>';
			}else{
				$html[] = 						'<td><span class="Instaberg_iw-error"><i class="fa fa-check-square"></i></span></td>';
			}
			$html[] = 							'<td width="30%"></td>';
			$html[] = 						'</tr>';
			$html[] = 						'<tr>';
			$html[] = 							'<td>' . esc_html__('Your Theme', 'instaberg') . '</td>';
			$template = get_option( 'template' );
			$themes = wp_get_themes();
			$theme = $themes[$template];

			$html[] = 							'<td><span class="Instaberg_iw-success"><i class="fa fa-check-square"></i></span></td>';
			$html[] = 							'<td><a href="' . ( esc_url( admin_url( '/themes.php' ) ) ) . '"><img width="80" src="' . ( esc_url( $theme->get_screenshot() ) ) . '" /></a></td>';
			$html[] = 						'</tr>';

			$html[] = 						'<tr>';
			$html[] = 							'<td width="55%">' . esc_html__('WP DEBUG', 'instaberg') . ':</td>';
			$html[] = 							'<td>' . (  defined('WP_DEBUG') && WP_DEBUG === true ? 'YES' : "NO" ) . '</td>';
			$html[] = 						'</tr>';

			//.Instaberg_iw-error | .Instaberg_iw-success
			$is_curl = (extension_loaded("curl") && function_exists('curl_init'));
			$css_class_curl = $is_curl ? 'Instaberg_iw-success' : 'Instaberg_iw-error';
			$html[] = 						'<tr>';
			$html[] = 							'<td width="55%">' . esc_html__('cURL enabled', 'instaberg') . ':</td>';
			$html[] = 							'<td class="' . $css_class_curl . '">' . (  $is_curl ? esc_html__('YES', 'instaberg') : esc_html__("NO", 'instaberg') ) . '</td>';
			$html[] = 						'</tr>';

			$is_sockets = (extension_loaded("sockets") && function_exists('fsockopen'));
			$css_class_sockets = $is_sockets ? 'Instaberg_iw-success' : 'Instaberg_iw-error';
			$html[] = 						'<tr>';
			$html[] = 							'<td width="55%">' . esc_html__('sockets enabled', 'instaberg') . ':</td>';
			$html[] = 							'<td class="' . $css_class_sockets . '">' . (  $is_sockets ? esc_html__('YES', 'instaberg') : esc_html__("NO", 'instaberg') ) . '</td>';
			$html[] = 						'</tr>';

			$html[] = 					'</table>';
			$html[] = 				'</div>';
			$html[] = 			'</div>';
			$html[] = 		'</div>';

			$html[] = 	'</div>';
			$html[] = '</div>';
			
			return implode( "\n", $html );
		}
	}
}