<?php
/**
 * Instaberg Main manager.
 *
 * @package Instaberg
 * @since   1.0
 */
if ( ! function_exists( 'Instaberg' ) ) {
	/**
	 * Instaberg manager.
	 * @since 1.0
	 * @return Instaberg
	 */
	function Instaberg() {
		global $Instaberg;

		return $Instaberg;
	}
}

if ( ! function_exists( 'Instaberg_Settings' ) ) {
	/**
	 * Instaberg manager.
	 * @since 1.0
	 * @return Instaberg_Settings
	 */
	function Instaberg_Settings() {
		return new Instaberg_Settings();
	}
}

if ( ! function_exists( 'Instaberg_Social' ) ) {
	/**
	 * Instaberg manager.
	 * @since 1.0
	 * @return Instaberg_Social
	 */
	function Instaberg_Social() {
		return new Instaberg_Social();
	}
}

if ( ! function_exists( 'Instaberg_Settings_Build_Options' ) ) {
	/**
	 * Instaberg manager.
	 * @since 1.0
	 * @return Instaberg_Settings
	 */
	function Instaberg_Settings_Build_Options( $options=array(), $box_id='' ) {
		return Instaberg_Settings()->build_options( $options, $box_id );
	}
}


if ( ! function_exists( 'Instaberg_Helper' ) ) {
	function Instaberg_Helper() {
		return new Instaberg_Helper();
	}
}

if ( ! function_exists( 'Instaberg_Woocommerce' ) ) {
	/**
	 * Instaberg manager.
	 * @since 1.0
	 * @return Instaberg_Settings
	 */
	function Instaberg_Woocommerce() {
		return new Instaberg_Woocommerce();
	}
}

if ( ! function_exists( 'Instaberg_Load_Options' ) ) {
	/**
	 * Instaberg manager.
	 * @since 1.0
	 * @return Instaberg_Settings
	 */
	function Instaberg_Load_Options( $dir ) {
	    new Instaberg_Settings(); // save settings first!
	    
		$options = array();
		// find if we have a options.php into the same folder
		$options_file = $dir . '/options.php';
		if( is_file($options_file) ){
			
			ob_start();
			require_once $options_file;
			
			$content = ob_get_contents();
			ob_clean();
 
			if( trim($content) != "" ){
				$options = json_decode( $content, true );
			}
		}
		return $options;
	}
}

if ( ! function_exists( 'Instaberg_mapper' ) ) {
	/**
	 * Shorthand for Instaberg Mapper.
	 * @since 1.0
	 * @return Instaberg_Mapper
	 */
	function Instaberg_mapper() {
		return Instaberg()->mapper();
	}
}
if ( ! function_exists( 'Instaberg_settings' ) ) {
	/**
	 * Shorthand for Instaberg settings.
	 * @since 1.0
	 * @return Instaberg_Settings
	 */
	function Instaberg_settings() {
		return Instaberg()->settings();
	}
}
if ( ! function_exists( 'Instaberg_path_dir' ) ) {
	/**
	 * Get file/directory path in Instaberg.
	 *
	 * @param string $name - path name
	 * @param string $file
	 *
	 * @since 1.0
	 * @return string
	 */
	function Instaberg_path_dir( $name, $file = '' ) {
		return Instaberg()->path( $name, $file );
	}
}
if ( ! function_exists( 'Instaberg_upload_dir' ) ) {
	/**
	 * Temporary files upload dir;
	 * @since 1.0
	 * @return string
	 */
	function Instaberg_upload_dir() {
		return Instaberg()->uploadDir();
	}
}
if ( ! function_exists( 'Instaberg_path_dir' ) ) {
	/**
	 * Get file/directory path in Instaberg.
	 *
	 * @param string $name - path name
	 * @param string $file
	 *
	 * @since 4.2
	 * @return string
	 */
	function Instaberg_path_dir( $name, $file = '' ) {
		return Instaberg()->path( $name, $file );
	}
}
if ( ! function_exists( 'Instaberg_post_param' ) ) {
	/**
	 * Get param value from $_POST if exists.
	 *
	 * @param $param
	 * @param $default
	 *
	 * @since 1.0
	 * @return null|string - null for undefined param.
	 */
	function Instaberg_post_param( $param, $default = null ) {
		return isset( $_POST[ $param ] ) ? $_POST[ $param ] : $default;
	}
}
if ( ! function_exists( 'Instaberg_get_param' ) ) {
	/**
	 * Get param value from $_GET if exists.
	 *
	 * @param $param
	 * @param $default
	 *
	 * @since 1.0
	 * @return null|string - null for undefined param.
	 */
	function Instaberg_get_param( $param, $default = null ) {
		return isset( $_GET[ $param ] ) ? $_GET[ $param ] : $default;
	}
}
if ( ! function_exists( 'Instaberg_request_param' ) ) {
	/**
	 * Get param value from $_REQUEST if exists.
	 *
	 * @param $param
	 * @param $default
	 *
	 * @since 4.4
	 * @return null|string - null for undefined param.
	 */
	function Instaberg_request_param( $param, $default = null ) {
		return isset( $_REQUEST[ $param ] ) ? $_REQUEST[ $param ] : $default;
	}
}
if ( ! function_exists( 'Instaberg_action' ) ) {
	/**
	 * Get Instaberg special action param.
	 * @since 1.0
	 * @return string|null
	 */
	function Instaberg_action() {
		$Instaberg_action = 'dashboard';
		if ( isset( $_GET['Instaberg_action'] ) ) {
			$Instaberg_action = $_GET['Instaberg_action'];
		} elseif ( isset( $_POST['Instaberg_action'] ) ) {
			$Instaberg_action = $_POST['Instaberg_action'];
		}

		return $Instaberg_action;
	}
}

if ( ! function_exists( 'Instaberg_asset_url' ) ) {
	/**
	 * Get full url for assets.
	 *
	 * @param string $file
	 *
	 * @since 4.2
	 * @return string
	 */
	function Instaberg_asset_url( $file ) {
		return Instaberg()->assetUrl( $file );
	}
}

if ( ! function_exists( 'Instaberg_templates_url' ) ) {
	/**
	 * Get full url for assets.
	 *
	 * @param string $file
	 *
	 * @since 4.2
	 * @return string
	 */
	function Instaberg_templates_url( $file ) {
		return Instaberg()->templatesUrl( $file );
	}
}

if ( ! function_exists( 'Instaberg_locate_template' ) ) {
	function Instaberg_locate_template( $template_name, $template_path = '', $default_path = '' ) 
	{
	    if ( ! $template_path ) {
	        $template_path = 'templates/';
	    }

	    if ( ! $default_path ) {
	        $default_path = Instaberg()->path( 'TEMPLATES_DIR', '' );
	    }
 
	    // Look within passed path within the theme - this is priority.
	    $template = locate_template(
	        array(
	            trailingslashit( $template_path ) . $template_name,
	            $template_name
	        )
	    );

	    // Get default template/
	    if ( ! $template ) {
	        $template = $default_path . $template_name;
	    }

	    // Return what we found.
	    return apply_filters( 'Instaberg_locate_template', $template, $template_name, $template_path );
	}
}

function Instaberg_plugin_name() {
	return Instaberg()->pluginName();
}

if ( ! function_exists( 'Instaberg_Instagram_Helper' ) ) {
	function Instaberg_Instagram_Helper() {
		if ( class_exists('Instaberg_Instagram_Helper') ) {
			return new Instaberg_Instagram_Helper();
		}
		return null;
	}
}





if ( ! function_exists( 'InstabergApiParse' ) ) {
	function InstabergApiParse() {
		if ( class_exists('Instaberg_Api_Parse') ) {
			return new Instaberg_Api_Parse();
		}
		return null;
	}
}

if ( ! function_exists( 'InstabergUtils' ) ) {
	function InstabergUtils() {
		if ( class_exists('Instaberg_Utils') ) {
			return new Instaberg_Utils();
		}
		return null;
	}
}