<?php
/**
 * Instagram API By Parsing Class
 * http://www.aa-team.com
 * ======================
 *
 * @package		Instagram API By Parsing Class
 * @author		George, AA-Team
 * @version		1.0
 */
!defined('ABSPATH') and exit;
if (class_exists('Instaberg_Api_Parse') != true) {
	class Instaberg_Api_Parse
	{
		const VERSION = '1.0';

		private $api_url = 'https://www.instagram.com/';
		private $api_info = array(
			'cookies_jar' 	=> array(), //received cookies!

			'headers' 		=> array(
				'User-Agent' 	=> 'Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/71.0.3578.98 Safari/537.36',
				'Origin' 		=> 'https://www.instagram.com',
				'Referer' 		=> 'https://www.instagram.com',
				'Connection' 	=> 'close'
			),

			'nodes' 		=> array(
				'get_post' 						=> '/p/%s/?__a=1', //{post_code}
				'get_igtv' 						=> '/tv/%s/?__a=1', //{post_code}
				'get_user'						=> '/%s/', //{username}
				'get_user_stories_media'		=> '/%s/', //{username}
				'get_user_media'				=> '/%s/', //{username}
				'get_tag_media' 				=> '/explore/tags/%s/', //{tag}
				'get_location_media'		 	=> '/explore/locations/%s/', //{location_id}
			),

			'validate_content_exp' => '/window\._sharedData\s?=\s?(.*);<\/script>/',

			//TODO: dynamically identify the hashes from requesting the js files!
			// 	it's something like ...queryId:"f2405b236d85e8296cf30347c9f08c2a",...
			'query_hash' 	=> array(

				// found in https://www.instagram.com/static/bundles/metro/ProfilePageContainer.js/0b1d520cb61e.js
				'get_user_media' 			=> 'f2405b236d85e8296cf30347c9f08c2a',

				// found in https://www.instagram.com/static/bundles/metro/Consumer.js/1db7a451b175.js
				'get_tag_media' 			=> 'f92f56d47dc7a55b606908374b43a314',

				// found in https://www.instagram.com/static/bundles/metro/LocationPageContainer.js/9556787219f5.js
				'get_location_media' 		=> '1b84447a4d8b6d6d0426fefb34514485',
			),

			'media_limits' 	=> array(
				'per_page' 		=> 50,
				'max' 			=> 250,
			),
		);

		private $settings = array();
		private $proxy = array();
		private $media_max = array();

		
		/**
		 * The constructor
		 */
		public function __construct()
		{
			$this->settings = get_option( 'Instaberg_Plugin_Settings' );
			$this->proxy = array(
				'server' 		=> isset($this->settings['proxy_server']) ? $this->settings['proxy_server'] : '',
				'username' 		=> isset($this->settings['proxy_username']) ? $this->settings['proxy_username'] : '',
				'password' 		=> isset($this->settings['proxy_password']) ? $this->settings['proxy_password'] : '',
			);
			$this->media_max = array(
				'get_user_media' 		=> isset($this->settings['media_max_user']) ? $this->settings['media_max_user'] : 75,
				'get_tag_media' 		=> isset($this->settings['media_max_tag']) ? $this->settings['media_max_tag'] : 100,
				'get_location_media' 	=> isset($this->settings['media_max_location']) ? $this->settings['media_max_location'] : 125,
			);

			// create AJAX request
			add_action('wp_ajax_Instaberg_InstagramApiParse', array(
				$this,
				'ajax_requests'
			));
		}

		//=========================================================================================
		// Ajax Requests
		public function ajax_requests()
		{
			$input = array(
				'id' 		=> isset($_REQUEST['id']) ? (string) $_REQUEST['id'] : '',
				'action' 	=> isset($_REQUEST['sub_action']) ? (string) $_REQUEST['sub_action'] : 'none',
				'how' 		=> isset($_REQUEST['how']) ? (string) $_REQUEST['how'] : '',
			);
			extract( $input );

			$ret = array(
				'is_cache'		=> false,
				'status' 		=> 'invalid',
				'msg' 			=> '',
				'msg_header' 	=> '',
				'data' 			=> '',
			);

			// api testing
			if ( in_array($how, array('run', 'rundev')) ) {

				$use_cache = 'yes';

				if ( ! in_array($action, array(
					'get_post',
					'get_igtv',
					'get_user',
					'get_user_media',
					'get_tag_media',
					'get_location_media',
				)) ) {
					$ret = array_replace_recursive( $ret, array(
						'msg' => sprintf('action %s is not allowed', $action),
					));
					die(json_encode($ret));
				}

				$opStat = $this->$action( $id, array(
					'use_cache' 	=> false, //'yes' === $use_cache ? true : false,
				));
				$ret = array_replace_recursive( $ret, $opStat );

				if ( 'run' === $how ) {

					$box_visual = '<span class="Instaberg-message Instaberg-error">' . $ret['msg'] . '</span>';
					if ( 'valid' == $ret['status'] ) {
						$box_visual = "_{$action}_box_visual";
						$box_visual = $this->$box_visual( $ret['data'] );
					}

					$ret = array_replace_recursive( $ret, array(
						'box_visual' 	=> $box_visual,
					));
				}
				else if ( 'rundev' === $how ) {
					var_dump('<pre>', $ret, '</pre>'); echo __FILE__ . ":" . __LINE__;die . PHP_EOL;
				}

				die(json_encode($ret));
			}

			if ( 'add_posts' === $action ) {

				$opStatList = $this->get_data_list( $id );
			}
			else if ( 'regenerate_posts' === $action ) {

				$id = $this->_string_to_array( $id );
				$id = array_map( function( $item ) {
					return "https://www.instagram.com/p/$item/";
				}, $id );

				$opStatList = $this->get_data_list( $id );
			}

			$ret = array_replace_recursive( $ret, array(
				'status' => 'valid',
				'msg' => $opStatList['msg'],
				'data' => $opStatList['data'],
			));
			unset( $ret['is_cache'], $ret['msg_header'] );

			die(json_encode($ret));
		}

		//=========================================================================================
		// request multiple instagram sources
		public function get_data_list( $id ) {

			$data = array();
			$msg = array();
			$ret = array(
				'data' => $data,
				'msg' => $msg,
			);

			// clean id if necessary
			$id = $this->_build_id_list( $id );
			if ( empty($id) ) {
				return array_replace_recursive( $ret, array(
					'msg' => 'id si empty!',
				));
			}

			// loop first
			$cc = 0;
			foreach ( $id as $idkey => $idval ) {

				if ( empty($idval) || ! is_array($idval) ) {
					continue 1;
				}

				// loop second
				foreach ( $idval as $idval2 ) {

					$action = "get_{$idkey}";

					$opStat = $this->$action( $idval2, array(
						'use_cache' 	=> true,
					));

					$msg[] = array( 'status' => $opStat['status'], 'msg' => $opStat['msg'] );

					$idval3 = $opStat['data'];
					if ( in_array($idkey, array('post', 'igtv')) ) {
						$idval3 = array( $opStat['data'] );
					}

					// loop third
					foreach ( $idval3 as $vv ) {

						$postkey = $vv['id'];
						if ( ! isset($data["$postkey"]) ) {
							$data["$postkey"] = $this->_get_data_list_format_item( $vv );
						}
					}
					// end loop third

					$cc++;
				}
				// end loop second
			}
			// end loop first

			$ret = array_replace_recursive( $ret, array(
				'data' => $data,
				'msg' => $msg,
			));

			return $ret;
		}

		private function _get_data_list_format_item( $data ) {

			if ( empty($data) || ! is_array($data) ) {
				return array();
			}

			$ret = array(
				'id' 				=> $data['id'],

				'userid' 			=> $data['user']['id'],
				'username' 			=> $data['user']['username'],
				'user_fullname' 	=> $data['user']['full_name'],
				'user_pic' 			=> $data['user']['profile_picture'],
				'post_code' 		=> $data['code'],
				'post_type' 		=> $data['type'],

				'tags' 				=> $data['tags'],
				'total_tags' 		=> count($data['tags']),

				'total_comments' 	=> isset($data['comments']['count'])
					? $data['comments']['count'] : 0,

				'total_likes' 	=> isset($data['likes']['count'])
					? $data['likes']['count'] : 0,

				'video_url' 		=> $data['video_url'],
				'link' 				=> $data['link'],

				'image' 			=> isset($data['images']['original']['url'])
					&& ! empty($data['images']['original']['url']) ? $data['images']['original']['url'] : '',
				'thumb' 			=> '',

				'caption' 			=> isset($data['caption']['text'])
					&& ! empty($data['caption']['text']) ? $data['caption']['text'] : '',
			);

			foreach ( array('medium', 'low', 'thumbnail') as $thumb_size ) {
				if ( isset($data['images']["$thumb_size"], $data['images']["$thumb_size"]['url'])
					&& ! empty($data['images']["$thumb_size"]['url'])
				) {
					$ret['thumb'] = $data['images']["$thumb_size"]['url'];
				}
			}
			return $ret;
		}

		private function _build_id_list( $id ) {

			$id = $this->_string_to_array( $id );
			if ( empty($id) ) {
				return $id;
			}

			//posts: https://www.instagram.com/p/Btdb3tmo2bm/
			//videos: https://www.instagram.com/tv/BuRnuq9Au5E/
			//instagram sources:
			// 	https://www.instagram.com/leomessi/
			// 	https://www.instagram.com/explore/tags/ourotherclub/
			// 	https://www.instagram.com/explore/locations/491944/marina-city/
			//instagram sources shortcuts:
			// 	@leomessi
			// 	#ourotherclub
			// 	&491944/marina-city

			$ret = array(
				'post' => array(),
				'igtv' => array(),
				'user_media' => array(),
				'tag_media' => array(),
				'location_media' => array(),
			);

			foreach ( $id as $vv ) {

				$regexp = "~https?://www\.instagram\.com/(?P<what>p/|tv/|explore/tags/|explore/locations/|)([^/]*)(?:/(?:.*)/?|/?)~imu";
				$found = preg_match_all( $regexp, $vv, $m );

				$opTmpStat = $this->_build_id_list_regexp_get( $m );
				if ( false !== $opTmpStat && ! in_array($opTmpStat[1], $ret["{$opTmpStat[0]}"]) ) {
					$ret["{$opTmpStat[0]}"][] = $opTmpStat[1];
				}
				else {
					$regexp = "~(?P<what>@|#|&)([^/]*)~imu";
					$found = preg_match_all( $regexp, $vv, $m );
					//var_dump('<pre>',$found, $m ,'</pre>');

					$opTmpStat = $this->_build_id_list_regexp_get( $m );
					if ( false !== $opTmpStat && ! in_array($opTmpStat[1], $ret["{$opTmpStat[0]}"]) ) {
						$ret["{$opTmpStat[0]}"][] = $opTmpStat[1];
					}
				}
			}

			return $ret;
		}

		private function _build_id_list_regexp_get( $m ) {

			$conv = array(
				'' => 'user_media',
				'p/' => 'post',
				'tv/' => 'igtv',
				'explore/tags/' => 'tag_media',
				'explore/locations/' => 'location_media',
				'@' => 'user_media',
				'#' => 'tag_media',
				'&' => 'location_media',
			);

			if ( is_array($m) && isset($m['what'], $m['what'][0], $m[2], $m[2][0]) && ! empty($m[2][0]) ) {

				$what = trim( $m['what'][0] );
				$what = isset($conv["$what"]) ? $conv["$what"] : '';

				$val = trim( $m[2][0] );

				if ( ! empty($what) ) {
					return array( $what, $val );
				}
			}
			return false;
		}



		//=========================================================================================
		// main function to make a request to the api

		private function make_request( $url_method, $http_method='GET', $params=array() ) {

			$params = array_replace_recursive( array(
				'query' 		=> array(),
				'headers' 		=> array(),
				'cookies' 		=> array(),
				'postfields' 	=> array(), //in case you make a post request
			), $params );

			$ret = array(
				'status' 		=> 'invalid',
				'msg' 			=> '',
				'http_code' 	=> 0,
				'data' 			=> ''
			);

			$can_do_request = $this->_make_request_can_do();
			if ( 'invalid' === $can_do_request['status'] ) {
				return array_replace_recursive( $ret, array(
					'msg' 	=> $can_do_request['msg'],
				));
			}


			//build request url
			$url = array();
			$url[] = rtrim( $this->api_url, '/' );
			$url[] = $url_method;
			$url = implode( '', $url );

			//get url by components
			$urlparts = $this->_make_request_parse_url( $url, $params['query'] );
			$url_curl = $urlparts['url_curl'];

			//build headers
			$get_headers = $this->_make_request_build_headers( $urlparts['host'], $params['headers'], $params['cookies'], $params['postfields'] );
			$headers = $get_headers['headers'];
			$postdata = $get_headers['postdata'];

			//:: curl request
			$output = array();
			$is_err = false;

			if ( in_array( $can_do_request['what'], array('curl', 'both') ) && ! $is_err ) {

				//request parameters (in & out)
				$input_params = array(
					'returntransfer' 	=> true,
					'header' 			=> true,
					'httpheader' 		=> $headers,
					'connecttimeout' 	=> 15,
					'timeout' 			=> 45,
					'postfields' 		=> $postdata,
				);
				if ( ! empty($this->proxy) ) {
					$input_params['proxy'] = $this->proxy['server'];
					if ( ! empty($this->proxy['username']) && ! empty($this->proxy['password']) ) {
						$input_params['proxy_userpwd'] = $this->proxy['username'] . ':' . $this->proxy['password'];
					}
				}
				if ( 'POST' == $http_method ) {
					$input_params = array_replace_recursive( $input_params, array(
						'post' 			=> true,
					));
				}

				$output_params = array(
					'parse_headers' 		=> true,
					'resp_is_json' 			=> false,
					'resp_add_http_code' 	=> false,
					'find_cookies' 			=> true,
				);

				$output = InstabergUtils()->curl( $url_curl, $input_params, $output_params, true );

				if ( 0 === $output['http_code'] ) {

					$is_err = true;

					if ( 'curl' === $can_do_request['what'] ) { //only curl can be used!
						return array_replace_recursive( $ret, $output );
					}
				}
			}

			//:: fsocket request /FALLBACK request
			if ( $is_err || 'fsocket' === $can_do_request['what'] ) {

				//request parameters (in & out)
				$input_params = array(
					'http_method' 		=> $http_method,
					'urlparts' 			=> $urlparts,
					'headers' 			=> $headers,
					'postfields' 		=> $postdata,
				);
				$output_params = array(
					'parse_headers' 		=> true,
					'resp_is_json' 			=> false,
					'resp_add_http_code' 	=> false,
					'find_cookies' 			=> true,
				);

				$output = InstabergUtils()->fsocket( $url_curl, $input_params, $output_params, true );
			}

			$ret = array_replace_recursive( $ret, $output );

			if ( 'invalid' === $ret['status'] || empty($ret['data']) ) {
				return $ret;
			}

			//:: set cookies (if needed)
			$ret['cookies'] = $this->_make_request_set_cookies( $ret['cookies'], $urlparts['host'] );

			return $ret;
		}

		private function _make_request_can_do() {

			$ret = array(
				'status' 	=> 'invalid',
				'what' 		=> 'none', //curl | fsocket | both | none
				'msg' 		=> '',
			);

			$is_curl = (extension_loaded("curl") && function_exists('curl_init'));
			$is_sockets = (extension_loaded("sockets") && function_exists('fsockopen'));

			if ( ! $is_curl && ! $is_sockets ) {
				return array_replace_recursive( $ret, array(
					'msg' 		=> esc_html__('cURL and fsockets aren\'t supported on your server!', 'instaberg'),
				));
			}
			else if ( ! $is_curl ) {
				return array_replace_recursive( $ret, array(
					'status' 	=> 'valid',
					'what' 		=> 'fsocket',
					'msg' 		=> esc_html__('cURL isn\'t supported on your server!', 'instaberg'),
				));
			}
			else if ( ! $is_sockets ) {
				return array_replace_recursive( $ret, array(
					'status' 	=> 'valid',
					'what' 		=> 'curl',
					'msg' 		=> esc_html__('fsockets isn\'t supported on your server!', 'instaberg'),
				));
			}

			return array_replace_recursive( $ret, array(
				'status' 	=> 'valid',
				'what' 		=> 'both',
				'msg' 	 	=> esc_html__('cURL and fsockets are supported on your server!', 'instaberg'),
			));
		}

		private function _make_request_parse_url( $url, $query_str='' ) {

			$url_info = parse_url($url);

			$scheme 	= isset($url_info['scheme']) && ! empty($url_info['scheme']) ? $url_info['scheme'] : '';
			$host 		= isset($url_info['host']) && ! empty($url_info['host']) ? $url_info['host'] : '';
			$port 		= isset($url_info['port']) && ! empty($url_info['port']) ? $url_info['port'] : '';
			$path 		= isset($url_info['path']) && ! empty($url_info['path']) ? $url_info['path'] : '';
			$query 		= isset($url_info['query']) && ! empty($url_info['query']) ? $url_info['query'] : '';

			if ( ! empty($query_str) ) {
				$query = http_build_query( $query_str );
			}

			//curl url
			$url_curl = array();
			$url_curl[] = $scheme;
			$url_curl[] = '://';
			$url_curl[] = $host;
			$url_curl[] = $path;
			$url_curl[] = !empty($query) ? '?' . $query : '';
			$url_curl = implode('', $url_curl);

			//fsocket url

			return compact('scheme', 'host', 'port', 'path', 'query', 'url_curl');
		}

		private function _make_request_build_headers( $host, $headers_arr=array(), $cookies_arr=array(), $postfields=array() ) {

			//begin headers
			$headers = $this->api_info['headers'];

			if ( ! empty($headers_arr) ) {
				$headers = array_replace_recursive($headers, $headers_arr);
			}

			$headers['Host'] = $host;

			//build cookies
			$cookies = $this->_make_request_get_current_cookies( $host );

			if ( ! empty($cookies_arr) ) {
				$cookies = array_replace_recursive($cookies, $cookies_arr);
			}

			if ( ! empty($cookies) ) {

				$cookies_ = array();

				foreach ($cookies as $cookie_name => $cookie_value) {
					$cookies_[] = $cookie_name . '=' . $cookie_value;
				}

				$headers['Cookie'] = implode('; ', $cookies_);
			}

			$postdata = '';
			if ( ! empty($postfields) ) {
				$postdata = http_build_query($postfields);
				$headers['Content-Type'] = 'application/x-www-form-urlencoded';
				$headers['Content-Length'] = strlen($postdata);

			}

			//build headers
			$headers_ = array();

			foreach ($headers as $header_key => $header_value) {
				$headers_[] = $header_key . ': ' . $header_value;
			}
			$headers = $headers_;

			return compact('headers', 'postdata');
		}

		private function _make_request_get_current_cookies( $host ) {
			$cookies_jar = $this->api_info['cookies_jar'];

			$cookies = isset($cookies_jar[$host]) && ! empty($cookies_jar[$host]) ? $cookies_jar[$host] : array();
			return $cookies;
		}

		private function _make_request_set_cookies( $cookies, $host ) {

			if ( ! empty($cookies) ) {

				$cookies = array_replace_recursive( $cookies, array(
					'ig_or' => 'landscape-primary',
					'ig_pr' => '1',
					'ig_vh' => rand(500, 1000),
					'ig_vw' => rand(1100, 2000),
				));

				$cookies_current = $this->_make_request_get_current_cookies( $host );
				$this->api_info['cookies_jar'][$host] = array_replace_recursive( $cookies_current, $cookies );
			}

			return $cookies;
		}

		private function make_request_do_query( $params=array() ) {

			$params = array_replace_recursive( array(
				'query_hash' 		=> array(),
				'page_data' 		=> array(),
				'variables' 		=> array(),
			), $params );
			extract( $params );

			$api_node = '/graphql/query/';

			$variables_json = json_encode( $variables );

			$csrf_token = isset($page_data['config']['csrf_token']) ? $page_data['config']['csrf_token'] : '';

			$rhx_gis = isset($page_data['rhx_gis']) ? $page_data['rhx_gis'] : '';

			$gis = implode( ':', array( $rhx_gis, $variables_json ) );
			$gis = md5( $gis );


			$page_result = $this->make_request( $api_node, 'GET', array(
				'query' => array(
					'query_hash' 	=> $query_hash,
					'variables' 	=> $variables_json,
				),
				'headers' => array(
					'X-Requested-With' 	=> 'XMLHttpRequest',
					'X-Instagram-Ajax' 	=> '1',
					'X-Instagram-Gis' 	=> $gis,
				),
			));
			return $page_result;
		}


		//=========================================================================================
		// PRIVATE

		private function _get_api_node( $node, $id ) {
			if ( isset($this->api_info['nodes']["$node"]) ) {
				return sprintf( $this->api_info['nodes']["$node"], $id );
			}
			return null;
		}

		private function _set_response_success( $ret, $msg='ok', $data=array() ) {
			$ret = array_replace_recursive( $ret, array(
				'status' 	=> 'valid',
			));
			if ( ! empty($msg) ) {
				$ret['msg'] = $msg;
			}
			if ( ! empty($ret['msg_header']) ) {
				$ret['msg'] = $ret['msg_header'] . $ret['msg'];
			}
			if ( ! empty($data) ) {
				$ret['data'] = $data;
			}
			return $ret;
		}
		private function _set_response_error( $ret, $msg='unknown error' ) {
			$ret = array_replace_recursive( $ret, array(
				'status' 	=> 'invalid',
			));
			if ( ! empty($msg) ) {
				$ret['msg'] = $msg;
			}
			if ( ! empty($ret['msg_header']) ) {
				$ret['msg'] = $ret['msg_header'] . $ret['msg'];
			}
			return $ret;
		}

		private function _is_valid_content( $content ) {
			return is_array($content) && ! empty($content);
		}

		private function _get_media_max( $api_method ) {

			$limit = isset($this->api_info['media_limits']['max']) ? (int) $this->api_info['media_limits']['max'] : 250;
			$limit = isset($this->media_max["$api_method"]) ? (int) $this->media_max["$api_method"] : $limit;

			return $limit;
		}

		private function _get_media_perpage( $api_method ) {

			$limit = isset($this->api_info['media_limits']['per_page']) ? (int) $this->api_info['media_limits']['per_page'] : 50;

			return $limit;
		}

		private function _media_data_formatted( $data=array(), $data_has=array() ) {

			$data_has = array_replace_recursive( array(
				'user_data' 	=> array(),
			), $data_has);

			$data_new = array();

			//:: user
			$user_data = array_replace_recursive( array(
				'id' 				=> '',
				'username' 			=> '',
				'profile_picture' 	=> '',
				'full_name' 		=> ''
			), $data_has['user_data']);

			if ( empty($user_data['id']) && isset($data['owner']) && ! empty($data['owner']) ) {
				$__ = $data['owner'];
				$user_data = array_replace_recursive( array(
					'id' 				=> isset($__['id']) ? $__['id'] : '',
					'username' 			=> isset($__['username']) && ! empty($__['username']) ? $__['username'] : '',
					'profile_picture' 	=> isset($__['profile_pic_url']) && ! empty($__['profile_pic_url']) ? $__['profile_pic_url'] : '',
					'full_name' 		=> isset($__['full_name']) && ! empty($__['full_name']) ? $__['full_name'] : ''
				));
			}

			//:: utils
			$u = array(
				'id' 					=> isset($data['id']) ? $data['id'] : '',

				'dim_w' 				=> isset($data['dimensions']['width']) ? $data['dimensions']['width'] : 0,
				'dim_h' 				=> isset($data['dimensions']['height']) ? $data['dimensions']['height'] : 0,

				'date' 					=> isset($data['date']) ? $data['date'] : '',
				'taken_at_timestamp' 	=> isset($data['taken_at_timestamp']) ? $data['taken_at_timestamp'] : '',

				'code' 					=> isset($data['code']) ? $data['code'] : '',
				'shortcode' 			=> isset($data['shortcode']) ? $data['shortcode'] : '',

				'is_video' 				=> isset($data['is_video']) && $data['is_video'] ? true : false,
				'__typename' 			=> isset($data['__typename']) ? $data['__typename'] : '',

				'thumbnail_src' 		=> isset($data['thumbnail_src']) ? $data['thumbnail_src'] : '',
				'display_src' 			=> isset($data['display_src']) ? $data['display_src'] : '',
				'display_url' 			=> isset($data['display_url']) ? $data['display_url'] : '',

				// post, igtv
				'display_resources' 	=> isset($data['display_resources']) ? $data['display_resources'] : array(),

				// user_media, tag_media, location_media
				'thumbnail_resources' 	=> isset($data['thumbnail_resources']) ? $data['thumbnail_resources'] : array(),

				'comments' 				=> isset($data['comments']) ? $data['comments'] : array(),
				'edge_media_to_comment' => isset($data['edge_media_to_comment']) ? $data['edge_media_to_comment'] : array(),

				'likes' 				=> isset($data['likes']) ? $data['likes'] : array(),
				'edge_liked_by' 		=> isset($data['edge_liked_by']) ? $data['edge_liked_by'] : array(),
				'edge_media_preview_like' => isset($data['edge_media_preview_like']) ? $data['edge_media_preview_like'] : array(),
			);
			$u['image_ratio'] = ! empty($u['dim_w']) && ! empty($u['dim_h']) ? $u['dim_h'] / $u['dim_w'] : 1;
			$u['display_img'] = ! empty($u['display_src']) ? $u['display_src'] : $u['display_url'];

			//:: general
			$data_new = array(
				'type' 			=> $u['is_video'] ? 'video' : ( 'GraphSidecar' === $u['__typename'] ? 'carousel' : 'image' ),
				'id' 			=> $u['id'] . ( ! empty($user_data['id']) ? '_' . $user_data['id'] : '' ),
				'code' 			=> ! empty($u['code']) ? $u['code'] : $u['shortcode'],

				'link' 			=> sprintf( 'https://www.instagram.com/p/%s/', (! empty($u['code']) ? $u['code'] : $u['shortcode']) ),
				'created_time' 	=> ! empty($u['date']) ? $u['date'] : $u['taken_at_timestamp'],

				'video_url' 	=> isset($data['video_url']) && ! empty($data['video_url']) ? $data['video_url'] : null,
				'video_duration'=> isset($data['video_duration']) && ! empty($data['video_duration']) ? $data['video_duration'] : null,
				'video_count' 	=> isset($data['video_view_count']) && ! empty($data['video_view_count']) ? $data['video_view_count'] : null,

				'user' 			=> $user_data,

				'location' 		=> array(),
				'caption' 		=> array(),

				'images' 		=> array(),
				'tags' 			=> array(),
				'comments' 		=> array(),
				'likes' 		=> array(),
			);

			//:: images
			$images = array();

			$image_sizes = array(
				//width => array( alias, width, height )
				150 	=> array( 'thumbnail', 150, 150 ),
				320 	=> array( 'low', 320, 320 ),
				640 	=> array( 'medium', 640, 640 ),
				750 	=> array( 'large', 750, 750 ),
				1080 	=> array( 'high', 1080, 1080 ),
			);

			foreach( $image_sizes as $image_key => $image_size ) {

				$url_ = $this->_media_image_resize( $u['display_img'], $image_size[1], $image_size[2] );
				$url_ = ''; // MEDIA IMAGE RESIZE DOESN'T SEEM TO WORK ANYMORE - 2019-june-07

				$images["{$image_size[0]}"] = array(
					'url' 			=> ! empty($u['thumbnail_src']) ? $u['thumbnail_src'] : $url_,
					'width' 		=> $image_size[1],
					'height' 		=> (int) ($u['image_ratio'] * $image_size[1]),
				);
			} // end foreach

			foreach( array('thumbnail_resources', 'display_resources') as $vv ) {

				if ( empty($u["$vv"]) || ! is_array($u["$vv"]) ) {
					continue 1;
				}

				$__ = $u["$vv"];

				foreach ( $__ as $image_info ) {

					$config_width = isset($image_info['config_width']) ? $image_info['config_width'] : 0;
					$config_height = isset($image_info['config_height']) ? $image_info['config_height'] : 0;

					if ( isset($image_sizes["$config_width"]) ) {

						$__2 = $image_sizes["$config_width"];

						$images["{$__2[0]}"] = array(
							'url' 			=> isset($image_info['src']) ? $image_info['src'] : null,
							'width' 		=> $config_width,
							'height' 		=> $config_height,
						);
					}
				} // end foreach

				if ( ! empty($__) ) {
					break;
				}
			} // end foreach

			$images['original'] = array(
				'url' 			=> $u['display_img'],
				'width' 		=> $u['dim_w'],
				'height' 		=> $u['dim_h'],
			);

			$data_new['images'] = $images;

			//:: location
			if ( isset($data['location']) && ! empty($data['location']) && is_array($data['location']) ) {

				$__ = $data['location'];
				$data_new['location'] = array(
					'name' 	=> isset($__['name']) ? $__['name'] : '',
					'id' 	=> isset($__['id']) ? $__['id'] : '',
				);
			}

			//:: tags & description
			$tags = array();
			if ( isset($data['caption']) && ! empty($data['caption']) ) {

				$__ = $data['caption'];
				$data_new['caption'] = array(
					'text' 			=> $__,
					'created_time' 	=> $data_new['created_time'],
					'from' 			=> $user_data,
				);

				$tags = $tags + $this->_media_parse_tags( $__ );
			}

			if ( isset($data['edge_media_to_caption']['edges'][0]['node']['text'])
				&& ! empty($data['edge_media_to_caption']['edges'][0]['node']['text'])
			) {
				$__ = $data['edge_media_to_caption']['edges'][0]['node']['text'];
				$data_new['caption'] = array(
					'text' 			=> $__,
					'created_time' 	=> $data_new['created_time'],
					'from' 			=> $user_data,
				);

				$tags = $tags + $this->_media_parse_tags( $__ );
			}

			$tags = array_unique( $tags );
			$tags = array_filter( $tags );

			$data_new['tags'] = $tags;

			//:: comments
			$comments = array();

			foreach( array('edge_media_to_comment', 'comments') as $vv ) {

				$nodes = array();

				if ( empty($u["$vv"]) || ! is_array($u["$vv"]) ) {
					continue 1;
				}

				$__ = $u["$vv"];

				$comments = array(
					'count' => isset($__['count']) && ! empty($__['count']) ? $__['count'] : 0,
					'data' 	=> array(),
				);

				if ( 'comments' === $vv ) {
					$nodes = isset($__['nodes']) && ! empty($__['nodes']) ? $__['nodes'] : array();
				}
				else {
					$nodes = isset($__['edges']) && ! empty($__['edges']) ? $__['edges'] : array();
				}

				// get latest only!
				if ( ! empty($nodes) ) {
					$nodes = array_slice( $nodes, -10, 10 );
				}

				foreach( $nodes as $node ) {

					$comment = $node;
					$owner = array();

					if ( 'comments' === $vv ) {
						$owner = isset($comment['user']) && ! empty($comment['user']) && is_array($comment['user'])
							? $comment['user'] : array();
					}
					else {
						$comment = isset($node['node']) && ! empty($node['node']) ? $node['node'] : array();

						$owner = isset($comment['owner']) && ! empty($comment['owner']) && is_array($comment['owner'])
							? $comment['owner'] : array();
					}

					$comment_author = array();

					if ( ! empty($owner) && is_array($owner) ) {
						$comment_author = array(
							'id' 				=> isset($owner['id']) ? $owner['id'] : '',
							'username' 			=> isset($owner['username']) ? $owner['username'] : '',
							'profile_picture' 	=> isset($owner['profile_pic_url']) ? $owner['profile_pic_url'] : '',
						);
					}

					$comments['data'][] = array(
						'created_time' 	=> isset($comment['created_at']) ? $comment['created_at'] : '',
						'text' 			=> isset($comment['text']) ? $comment['text'] : '',
						'from' 			=> $comment_author
					);
				} // end foreach

				if ( ! empty($nodes) ) {
					break;
				}
			} // end foreach

			$data_new['comments'] = $comments;

			//:: likes
			$likes = array();

			foreach( array('edge_media_preview_like', 'edge_liked_by', 'likes') as $vv ) {

				$nodes = array();

				if ( empty($u["$vv"]) || ! is_array($u["$vv"]) ) {
					continue 1;
				}

				$__ = $u["$vv"];

				$likes = array(
					'count' => isset($__['count']) && ! empty($__['count']) ? $__['count'] : 0,
					'data' 	=> array(),
				);

				if ( 'likes' === $vv ) {
					$nodes = isset($__['nodes']) && ! empty($__['nodes']) ? $__['nodes'] : array();
				}
				else {
					$nodes = isset($__['edges']) && ! empty($__['edges']) ? $__['edges'] : array();
				}

				// get latest only!
				if ( ! empty($nodes) ) {
					$nodes = array_slice( $nodes, -10, 10 );
				}

				foreach( $nodes as $node ) {

					$owner = array();

					if ( 'likes' === $vv ) {
						$owner = isset($node['user']) && ! empty($node['user']) && is_array($node['user'])
							? $node['user'] : array();
					}
					else {
						$owner = isset($node['node']) && ! empty($node['node']) && is_array($node['node'])
							? $node['node'] : array();
					}

					$like_author = array();

					if ( ! empty($owner) && is_array($owner) ) {
						$like_author = array(
							'id' 				=> isset($owner['id']) ? $owner['id'] : '',
							'username' 			=> isset($owner['username']) ? $owner['username'] : '',
							'profile_picture' 	=> isset($owner['profile_pic_url']) ? $owner['profile_pic_url'] : '',
						);
					}

					$likes['data'][] = $like_author;
				} // end foreach

				if ( ! empty($nodes) ) {
					break;
				}
			} // end foreach

			$data_new['likes'] = $likes;

			return $data_new;
		}

		private function _media_data_is_valid( $data=array() ) {

			if ( isset($data['id']) && ! empty($data['id']) ) {
				return true;
			}
			return false;
		}

		private function _media_image_resize( $url, $w, $h ) {

			$regexp = array();
			$regexp[0] = '/\/s\d+x\d+\//';
			$regexp[1] = '/\/e\d+\//';
			$regexp[2] = '/(\.com\/[^\/]+)\//';
			$regexp_ = '/\/vp\//';

			$ret = null;
			$size = $w.'x'.$h;

			if ( preg_match($regexp[0], $url) ) {
				$ret = preg_replace($regexp[0], '/s' . $size . '/', $url);
				$ret = preg_replace($regexp_, '/', $ret);
			}
			else if ( preg_match($regexp[1], $url) ) {
				$ret = preg_replace($regexp[1], '/s' . $size . '/e$1/', $url);
				$ret = preg_replace($regexp_, '/', $ret);
			}
			else if ( preg_match($regexp[2], $url) ) {
				$ret = preg_replace($regexp[2], '$1/s' . $size . '/', $url);
				$ret = preg_replace($regexp_, '/', $ret);
			}
			return $ret;
		}

		private function _media_parse_tags( $text ) {
			preg_match_all('/#([\w_]+)/u', $text, $m);
			return isset($m[1]) ? $m[1] : array();
		}

		private function _do_query_set_page( $page_result=array() ) {
			$current_page = array(
				'status' 	=> isset($page_result['status']) ? $page_result['status'] : '...',
				'msg' 		=> isset($page_result['msg']) ? $page_result['msg'] : '...',
				'http_code' => isset($page_result['http_code']) ? $page_result['http_code'] : '...',
			);
			return $current_page;
		}


		//=========================================================================================
		// api_generic_methods

		private function api_generic_method( $id, $params=array() ) {

			$params = array_replace_recursive( array(
				'use_cache' 	=> true,
				'api_method' 	=> '',
				'api_node' 		=> '',
				'cache_type' 	=> '',
				'cache_name' 	=> '',
			), $params );
			extract( $params );

			$ret = array(
				'is_cache'		=> false,
				'status' 		=> 'invalid',
				'msg' 			=> '',
				'msg_header' 	=> '',
				'data' 			=> '',
			);

			$func_data_parsed = "_{$api_method}_data_parsed";
			$func_data_formatted = "_{$api_method}_data_formatted";

			$msg_header = "api node {$cache_type} ; $cache_name : ";
			$ret['msg_header'] = $msg_header;

			if ( empty($api_node) ) {
				return $this->_set_response_error( $ret, 'api node does\'t exist' );
			}

			//:: cached
			$save_cache = false;
			$content = null;
			if ( $use_cache ) {
				$content = InstabergUtils()->request_get_cache( $cache_name, $cache_type, array() );
				if ( $this->_is_valid_content( $content ) ) {
					$ret = array_replace_recursive( $ret, array(
						'is_cache' 	=> true,
						'data' 		=> $content,
					));
				}
			}

			//:: need request?
			if ( ! $this->_is_valid_content( $content ) ) {

				$page_result = $this->make_request( $api_node, 'GET', array() );

				if ( 'invalid' === $page_result['status'] || empty($page_result['http_code']) ) {
					return $this->_set_response_error( $ret, $page_result['msg'] );
				}

				switch ( $page_result['http_code'] ) {

					case 404:
						return $this->_set_response_error( $ret, sprintf('%s not found', $id) );
						break;

					case 200:
						$content_parsed = $this->$func_data_parsed( $page_result['data'], array(
							'id' 			=> $id,
							'api_method' 	=> $api_method,
							'msg_header' 	=> $msg_header,
							'page_result' 	=> $page_result,
						));

						$ret = array_replace( $ret, $content_parsed ); //don't use array_replace_recursive
						$content = $ret['data'];

						if ( 'invalid' === $ret['status'] ) {
							return $ret;
						}

						$save_cache = true;
						break;

					default:
						return $this->_set_response_error( $ret, sprintf('response http code seems invalid = %s', $page_result['http_code']) );
				}
			}

			//:: do we have content?
			if ( ! $this->_is_valid_content( $content ) ) {
				return $this->_set_response_error( $ret, 'content doesn\'t seem valid' );
			}

			$content_formatted = $this->$func_data_formatted( $content, array(
				'msg_header' 	=> $msg_header,
			));

			$ret = array_replace( $ret, $content_formatted ); //don't use array_replace_recursive

			if ( $save_cache && 'valid' === $ret['status'] ) {
				InstabergUtils()->request_save_cache( $cache_name, $cache_type, array(
					'response' 	=> $content,
				));

				$limit = $this->_get_media_max( $api_method );
				$ret['data'] = array_slice($ret['data'], 0, $limit);
			}

			return $ret;
		}

		// recursive method!
		private function api_do_query( $params=array() ) {

			static $ret, $cc;

			$params = array_replace_recursive( array(
				'_init' 		=> false,
				'api_method' 	=> '',
				'msg_header' 	=> '',
				'variables' 	=> array(),
				'cursor' 		=> 0,
				'nodes' 		=> array(),
				'pages' 		=> array(),
				'page_data' 	=> array(),
				'main_data' 	=> array(),
			), $params );
			extract( $params );

			if ( $_init ) {
				$ret = array(
					'status' 		=> 'invalid',
					'msg' 			=> '',
					'msg_header' 	=> isset($msg_header) ? $msg_header : '',
					'nodes' 		=> $nodes,
					'pages' 		=> $pages,
				);
				$cc = 0;
			}

			$limit = $this->_get_media_max( $api_method );

			$first = $this->_get_media_perpage( $api_method );
			if ( ! isset($variables['first']) ) {
				$variables['first'] = $first;
			}

			$cursor_start = $cursor;
			$cursor_next = $cursor_start + $variables['first'] - 1;

			//:: make request
			$query_hash = $this->api_info['query_hash']["$api_method"];

			$query_result = $this->make_request_do_query( array(
				'query_hash' 		=> $query_hash,
				'page_data' 		=> $page_data,
				'variables' 		=> $variables,
			));

			$current_page = $this->_do_query_set_page( $query_result );


			//:: parse request
			if ( 'invalid' === $query_result['status'] || 200 !== $query_result['http_code'] ) {
				$__ = sprintf('do query cursor %d-%d - request error!', $cursor_start, $cursor_next);

				if ( 400 === $query_result['http_code'] ) {
					$query_data = json_decode( $query_result['data'], true );
					if ( $query_data && is_array($query_data) && isset($query_data['message']) ) {
						$__ = sprintf('do query cursor %d-%d - %s!', $cursor_start, $cursor_next, $query_data['message']);
					}
				}

				$current_page = $this->_set_response_error( $current_page, $__ );
				$pages[] = $current_page;
				$ret['pages'] = $pages;

				$ret = $this->_set_response_error( $ret, $__ );
				return $ret;
			}

			$query_data = json_decode( $query_result['data'], true );

			if (
				! $query_data
				|| ! isset($query_data['data'])
				|| empty($query_data['data'])
			 ) {
				$__ = sprintf('do query cursor %d-%d - can\'t find data!', $cursor_start, $cursor_next);

				$current_page = $this->_set_response_error( $current_page, $__ );
				$pages[] = $current_page;
				$ret['pages'] = $pages;

				$ret = $this->_set_response_error( $ret, $__ );
				return $ret;
			}

			$query_data = $query_data['data'];

			//:: get media nodes
			switch ( $api_method ) {

				case 'get_user_media':

					$main_data['edge_owner_to_timeline_media']['edges'] = array();
					if ( isset($query_data['user']['edge_owner_to_timeline_media']['edges']) ) {
						$main_data['edge_owner_to_timeline_media']['edges'] = $query_data['user']['edge_owner_to_timeline_media']['edges'];
					}

					if ( isset($query_data['user']['edge_owner_to_timeline_media']['page_info']['end_cursor']) ) {
						$variables['after'] = $query_data['user']['edge_owner_to_timeline_media']['page_info']['end_cursor'];
					}
					
					break;

				case 'get_tag_media':

					$main_data['edge_hashtag_to_media']['edges'] = array();
					if ( isset($query_data['hashtag']['edge_hashtag_to_media']['edges']) ) {
						$main_data['edge_hashtag_to_media']['edges'] = $query_data['hashtag']['edge_hashtag_to_media']['edges'];
					}

					if ( isset($query_data['hashtag']['edge_hashtag_to_media']['page_info']['end_cursor']) ) {
						$variables['after'] = $query_data['hashtag']['edge_hashtag_to_media']['page_info']['end_cursor'];
					}

					break;

				case 'get_location_media':

					$main_data['edge_location_to_media']['edges'] = array();
					if ( isset($query_data['location']['edge_location_to_media']['edges']) ) {
						$main_data['edge_location_to_media']['edges'] = $query_data['location']['edge_location_to_media']['edges'];
					}

					if ( isset($query_data['location']['edge_location_to_media']['page_info']['end_cursor']) ) {
						$variables['after'] = $query_data['location']['edge_location_to_media']['page_info']['end_cursor'];
					}

					break;
			}

			$nodes_new = $this->api_get_media_get_nodes( $main_data, array(
				'api_method' 	=> $api_method,
			));

			if ( empty($nodes_new) ) {
				$__ = sprintf('do query cursor %d-%d - no posts found!', $cursor_start, $cursor_next);

				$current_page = $this->_set_response_error( $current_page, $__ );
				$pages[] = $current_page;
				$ret['pages'] = $pages;

				$ret = $this->_set_response_error( $ret, $__ );
				return $ret;
			}
			$nodes = array_replace_recursive( $nodes, $nodes_new );
			$ret['nodes'] = $nodes;

			//:: prepare parameters for another request
			$cursor_start = $cursor;

			$count = count($nodes_new);
			$cursor += $count;

			$cursor_next = $cursor - 1;

			if ( $count < $variables['first'] || $cursor >= $limit ) {
				$__ = sprintf('do query cursor %d-%d - no more posts or max limit reached!', $cursor_start, $cursor_next);

				$current_page = $this->_set_response_success( $current_page, $__ );
				$pages[] = $current_page;
				$ret['pages'] = $pages;

				$ret = $this->_set_response_success( $ret, $__ );
				return $ret;
			}
			else {

				$__ = sprintf('do query cursor %d-%d - ok!', $cursor_start, $cursor_next);

				$current_page = $this->_set_response_success( $current_page, $__ );
				$pages[] = $current_page;
				$ret['pages'] = $pages;

				$ret = $this->_set_response_success( $ret, $__ );

				//simulate browser
				usleep( rand(1, 5) * 500000 ); //microseconds

				//:: recursivity
				$this->api_do_query( array(
					'_init' 		=> false,
					'api_method' 	=> $api_method,
					'msg_header' 	=> $msg_header,
					'variables' 	=> $variables,
					'cursor' 		=> $cursor,
					'nodes' 		=> $nodes,
					'pages' 		=> $pages,
					'page_data' 	=> $page_data,
					'main_data' 	=> $main_data,
				));
			}

			return $ret;
		}

		private function api_get_media_data_parsed( $content, $params=array() ) {

			$params = array_replace_recursive( array(
				'id' 			=> '',
				'api_method' 	=> '',
				'msg_header' 	=> '',
				'page_result' 	=> array(),
			), $params );
			extract( $params );

			$ret = array(
				'status' 		=> 'invalid',
				'msg' 			=> '',
				'msg_header' 	=> isset($msg_header) ? $msg_header : '',
				'data' 			=> '',
			);

			$matches = array();

			$validate = preg_match( $this->api_info['validate_content_exp'], $content, $matches );

			if ( ! $validate || ! isset($matches[1]) ) {
				return $this->_set_response_error( $ret, 'page content - json not found!' );
			}

			$page_data = json_decode( $matches[1], true );

			$variables = array();
			$main_data = null;

			//:: get media nodes
			switch ( $api_method ) {

				case 'get_user_media':

					if (
						! $page_data
						|| ! isset($page_data['entry_data']['ProfilePage'][0]['graphql']['user'])
						|| empty($page_data['entry_data']['ProfilePage'][0]['graphql']['user'])
					 ) {
						return $this->_set_response_error( $ret, 'page content - can\'t find user profile!' );
					}

					$main_data = $page_data['entry_data']['ProfilePage'][0]['graphql']['user'];

					if ( isset($main_data['is_private']) && $main_data['is_private'] ) {
						return $this->_set_response_error( $ret, 'page content - user info is private!' );
					}

					$variables['id'] = isset($main_data['id']) ? $main_data['id'] : 0;
					if ( $main_data['edge_owner_to_timeline_media']['page_info']['end_cursor'] ) {
						$variables['after'] = $main_data['edge_owner_to_timeline_media']['page_info']['end_cursor'];
					}

					break;

				case 'get_tag_media':

					if (
						! $page_data
						|| ! isset($page_data['entry_data']['TagPage'][0]['graphql']['hashtag'])
						|| empty($page_data['entry_data']['TagPage'][0]['graphql']['hashtag'])
					 ) {
						return $this->_set_response_error( $ret, 'page content - can\'t find hashtag info!' );
					}

					$main_data = $page_data['entry_data']['TagPage'][0]['graphql']['hashtag'];

					$variables['tag_name'] = $id;
					if ( $main_data['edge_hashtag_to_media']['page_info']['end_cursor'] ) {
						$variables['after'] = $main_data['edge_hashtag_to_media']['page_info']['end_cursor'];
					}

					break;

				case 'get_location_media':

					if (
						! $page_data
						|| ! isset($page_data['entry_data']['LocationsPage'][0]['graphql']['location'])
						|| empty($page_data['entry_data']['LocationsPage'][0]['graphql']['location'])
					 ) {
						return $this->_set_response_error( $ret, 'page content - can\'t find location info!' );
					}

					$main_data = $page_data['entry_data']['LocationsPage'][0]['graphql']['location'];

					$variables['id'] = $id;
					if ( $main_data['edge_location_to_media']['page_info']['end_cursor'] ) {
						$variables['after'] = $main_data['edge_location_to_media']['page_info']['end_cursor'];
					}

					break;
			}

			//:: get media nodes
			$nodes = $this->api_get_media_get_nodes( $main_data, array(
				'api_method' 	=> $api_method,
			));
			if ( empty($nodes) ) {
				return $this->_set_response_error( $ret, 'page content - no posts found!' );
			}

			$cursor = count($nodes);

			//:: init pages
			$current_page = $this->_do_query_set_page( $page_result );
			$__ = sprintf('do query cursor %d-%d - ok!', 0, $cursor-1);
			$current_page = $this->_set_response_success( $current_page, $__ );

			$pages = array();
			$pages[] = $current_page;


			//:: recursive requesting pages
			$query_params = array(
				'_init' 		=> true,
				'api_method' 	=> $api_method,
				'msg_header' 	=> $msg_header,
				'variables' 	=> $variables,
				'cursor' 		=> $cursor,
				'nodes' 		=> $nodes,
				'pages' 		=> $pages,
				'page_data' 	=> $page_data,
				'main_data' 	=> $main_data,
			);
			$do_query_stat = $this->api_do_query( $query_params );

			//DEBUG
			if (0) {
				unset( $do_query_stat['page_data'], $do_query_stat['main_data'] );
				$do_query_stat['nodes'] = array_unique(array_keys($do_query_stat['nodes']));
				var_dump('<pre>do_query_stat', $do_query_stat , '</pre>'); echo __FILE__ . ":" . __LINE__;die . PHP_EOL;
			}

			$ret = array_replace_recursive( $ret, $do_query_stat );
			$ret['data'] = $ret['nodes'];
			unset( $ret['nodes'] );
			return $ret;
		}

		private function api_get_media_data_formatted( $data, $params=array() ) {

			$params = array_replace_recursive( array(
				'msg_header' 	=> '',
			), $params );
			extract( $params );

			$ret = array(
				'status' 		=> 'invalid',
				'msg' 			=> '',
				'msg_header' 	=> isset($msg_header) ? $msg_header : '',
				'data' 			=> '',
			);

			$data_new = $data;

			$is_valid = is_array($data_new) && ! empty($data_new) ? true : false;

			if ( ! $is_valid ) {
				return $this->_set_response_error( $ret, 'data formatted - invalid media data!' );
			}

			return $this->_set_response_success( $ret, sprintf( 'data formatted ok! %d posts found.', count($data_new) ), $data_new );
		}

		private function api_get_media_get_nodes( $data, $params=array() ) {

			$params = array_replace_recursive( array(
				'api_method' 	=> '',
			), $params );
			extract( $params );

			$data_new = array();

			$user_data = array();
			if ( isset($data['username']) ) {
				$user_data = array(
					'id' 				=> isset($data['id']) ? $data['id'] : 0,

					'username' 			=> isset($data['username']) ? $data['username'] : '',

					'profile_picture' 	=> isset($data['profile_pic_url']) ? $data['profile_pic_url'] : '',

					'full_name' 		=> isset($data['full_name']) ? $data['full_name'] : '',
				);
			}

			//:: get media nodes
			switch ( $api_method ) {

				case 'get_user_media':
					$edges = isset($data['edge_owner_to_timeline_media']['edges'])
						? $data['edge_owner_to_timeline_media']['edges'] : array();
					break;

				case 'get_tag_media':
					$edges = isset($data['edge_hashtag_to_media']['edges'])
						? $data['edge_hashtag_to_media']['edges'] : array();
					break;

				case 'get_location_media':
					$edges = isset($data['edge_location_to_media']['edges'])
						? $data['edge_location_to_media']['edges'] : array();
					break;
			}

			if ( empty($edges) || ! is_array($edges) ) {
				return $data_new;
			}

			$nodes = array();
			foreach( $edges as $edge ) {
				if ( isset($edge['node']) ) {
					$nodes[] = $edge['node'];
				}
			}

			foreach( $nodes as $media_node ) {
				$media_info = $this->_media_data_formatted( $media_node, array(
					'user_data' => $user_data
				));

				if ( $this->_media_data_is_valid( $media_info ) ) {

					$media_id = $media_info['id'];
					$data_new["$media_id"] = $media_info;
				}
			}

			return $data_new;
		}

		private function api_get_media_box_visual( $data ) {

			$first = array_slice($data, 0, 5, true);
			$last = array_slice($data, -5, 5, true);
			$choosen = $first + $last;

			$boxdata = array();
			foreach ( $choosen as $vv ) {
				$boxdata[] = array(
					'link' 		=> $vv['link'],
					'comments'	=> isset($vv['comments']['count']) ? $vv['comments']['count'] : 0,
					'image' 	=> isset($vv['images']['original']['url']) ? $vv['images']['original']['url'] : '',
					'text' 		=> isset($vv['caption']['text']) ? $vv['caption']['text'] : '',
				);
			}

			$len = count($boxdata);

			ob_start();
		?>
			<ul class="method_get_user_media">
				<li>
					Total media: <?php echo count($data); ?>
				</li>

				<?php
				$html = array();
				$cc = 0; $pause_at = 5;
				foreach ($boxdata as $val) {

					if ( $cc && ! ($cc % $pause_at) ) {
						$html[] = '<li>';
						$html[] = 	'..........................................';
						$html[] = '</li>';
					}

					$html[] = '<li>';
					$html[] = 	'<a href="' . $val['link'] . '" target="_blank">';
					$html[] = 		'<img src="' . $val['image'] . '" title="' . $val['text'] . '" />';
					$html[] = 	'</a>';
					$html[] = 	'<div>comments: ' . $val['comments'] . '</div>';
					$html[] = '</li>';

					$cc++;
				}
				$html = implode( PHP_EOL, $html );
				echo wp_kses($html, array(
				    'a' => array(
				        'href' => array(),
				        'target' => array()
				    ),
				    'li' => array(),
				    'img' => array(
				    	'src' => array(),
				    	'title' => array()
				    ),
				    'div' => array(),
				    'p' => array()
				));
				?>
			</ul>
		<?php
			return ob_get_clean();
		}


		//=========================================================================================
		// get_post
		public function get_post( $post_code, $params=array() ) {

			$params = array_replace_recursive( array(
				'use_cache' 	=> true,
			), $params );
			extract( $params );

			$api_method = 'get_post';
			$api_node = $this->_get_api_node( $api_method, $post_code );
			$cache_type = 'get_post';
			$cache_name = "post@{$post_code}";

			return $this->api_generic_method( $post_code, array(
				'use_cache' 	=> $use_cache,
				'api_method' 	=> $api_method,
				'api_node' 		=> $api_node,
				'cache_type' 	=> $cache_type,
				'cache_name' 	=> $cache_name,
			));
		}

		private function _get_post_data_parsed( $content, $params=array() ) {

			$params = array_replace_recursive( array(
				'msg_header' 	=> '',
				'page_result' 	=> array(),
			), $params );
			extract( $params );

			$ret = array(
				'status' 		=> 'invalid',
				'msg' 			=> '',
				'msg_header' 	=> isset($msg_header) ? $msg_header : '',
				'data' 			=> '',
			);

			$page_data = json_decode( $content, true );

			$post_media_data = null;

			if ( ! $page_data ) ;
			else if ( isset($page_data['entry_data']['PostPage'][0]['media'])
				 && ! empty($page_data['entry_data']['PostPage'][0]['media'])
			) {
				$post_media_data = $page_data['entry_data']['PostPage'][0]['media'];
			}
			else if ( isset($page_data['graphql']['shortcode_media'])
				 && ! empty($page_data['graphql']['shortcode_media'])
			) {
				$post_media_data = $page_data['graphql']['shortcode_media'];
			}

			if ( empty($post_media_data) ) {
				return $this->_set_response_error( $ret, 'page content - can\'t find post media!' );
			}
			return $this->_set_response_success( $ret, 'parsed ok!', $post_media_data );
		}

		private function _get_post_data_formatted( $data, $params=array() ) {

			$params = array_replace_recursive( array(
				'msg_header' 	=> '',
			), $params );
			extract( $params );

			$ret = array(
				'status' 		=> 'invalid',
				'msg' 			=> '',
				'msg_header' 	=> isset($msg_header) ? $msg_header : '',
				'data' 			=> '',
			);

			$data_new = $this->_media_data_formatted( $data );
			$is_valid = $this->_media_data_is_valid( $data_new );

			if ( ! $is_valid ) {
				return $this->_set_response_error( $ret, 'data formatted - invalid media!' );
			}
			return $this->_set_response_success( $ret, 'data formatted ok!', $data_new );
		}

		private function _get_post_box_visual( $data ) {

			$boxdata = array(
				'userid' 			=> $data['user']['id'],
				'username' 			=> $data['user']['username'],
				'user full name' 	=> $data['user']['full_name'],
				'postid_userid' 	=> $data['id'],
				'post code' 		=> $data['code'],
				'post type' 		=> $data['type'],
				'tags' 				=> count($data['tags']),
				'total comments' 	=> isset($data['comments']['count']) ? $data['comments']['count'] : 0,
			);
			if ( count($data['tags']) ) {
				$boxdata['tags'] .= ': ' . implode(' ', array_map( function($v) { return '#' . $v; }, $data['tags']));
			}

			ob_start();
		?>
			<ul class="method_get_post <?php echo esc_attr($data['type']); ?>">
				<li>
					<img src="<?php echo esc_url($data['images']['original']['url']); ?>" />
				</li>

				<?php
				$html = array();
				foreach ($boxdata as $key => $val) {
					$html[] = '<li>';
					$html[] = 	"<div>$key:</div>";
					$html[] = 	"<div>$val</div>";
					$html[] = '</li>';
				}
				$html = implode( PHP_EOL, $html );
				echo wp_kses($html, array(
					'li' => array(),
					'div' => array()
				));
				?>

				<?php if ( 'video' === $data['type'] ) { ?>
				<li>
					<video width="600" controls>
						<source src="<?php echo esc_url($data['video_url']); ?>" type="video/mp4">
						Your browser does not support HTML5 video.
					</video>
				</li>
				<?php } ?>
			</ul>
		<?php
			return ob_get_clean();
		}


		//=========================================================================================
		// get_igtv
		public function get_igtv( $post_code, $params=array() ) {

			$params = array_replace_recursive( array(
				'use_cache' 	=> true,
			), $params );
			extract( $params );

			$api_method = 'get_igtv';
			$api_node = $this->_get_api_node( $api_method, $post_code );
			$cache_type = 'get_igtv';
			$cache_name = "igtv@{$post_code}";

			return $this->api_generic_method( $post_code, array(
				'use_cache' 	=> $use_cache,
				'api_method' 	=> $api_method,
				'api_node' 		=> $api_node,
				'cache_type' 	=> $cache_type,
				'cache_name' 	=> $cache_name,
			));
		}

		private function _get_igtv_data_parsed( $content, $params=array() ) {

			return $this->_get_post_data_parsed( $content, $params );
		}

		private function _get_igtv_data_formatted( $data, $params=array() ) {

			return $this->_get_post_data_formatted( $data, $params );
		}

		private function _get_igtv_box_visual( $data ) {

			return $this->_get_post_box_visual( $data );
		}


		//=========================================================================================
		// get_user
		public function get_user( $username, $params=array() ) {

			$params = array_replace_recursive( array(
				'use_cache' 	=> true,
			), $params );
			extract( $params );

			$api_method = 'get_user';
			$api_node = $this->_get_api_node( $api_method, $username );
			$cache_type = 'get_user';
			$cache_name = "user@{$username}_profile";

			return $this->api_generic_method( $username, array(
				'use_cache' 	=> $use_cache,
				'api_method' 	=> $api_method,
				'api_node' 		=> $api_node,
				'cache_type' 	=> $cache_type,
				'cache_name' 	=> $cache_name,
			));
		}

		private function _get_user_data_parsed( $content, $params=array() ) {

			$params = array_replace_recursive( array(
				'msg_header' 	=> '',
				'page_result' 	=> array(),
			), $params );
			extract( $params );

			$ret = array(
				'status' 		=> 'invalid',
				'msg' 			=> '',
				'msg_header' 	=> isset($msg_header) ? $msg_header : '',
				'data' 			=> '',
			);

			$matches = array();

			$validate = preg_match( $this->api_info['validate_content_exp'], $content, $matches );

			if ( ! $validate || ! isset($matches[1]) ) {
				return $this->_set_response_error( $ret, 'page content - json not found!' );
			}

			$page_data = json_decode( $matches[1], true );

			if (
				! $page_data
				|| ! isset($page_data['entry_data']['ProfilePage'][0]['graphql']['user'])
				|| empty($page_data['entry_data']['ProfilePage'][0]['graphql']['user'])
			 ) {
				return $this->_set_response_error( $ret, 'page content - can\'t find user profile!' );
			}

			$user_data = $page_data['entry_data']['ProfilePage'][0]['graphql']['user'];

			if ( isset($user_data['is_private']) && $user_data['is_private'] ) {
				return $this->_set_response_error( $ret, 'page content - user info is private!' );
			}

			return $this->_set_response_success( $ret, 'parsed ok!', $user_data );
		}

		private function _get_user_data_formatted( $data, $params=array() ) {

			$params = array_replace_recursive( array(
				'msg_header' 	=> '',
			), $params );
			extract( $params );

			$ret = array(
				'status' 		=> 'invalid',
				'msg' 			=> '',
				'msg_header' 	=> isset($msg_header) ? $msg_header : '',
				'data' 			=> '',
			);

			$data_new = array(
				'id' 				=> isset($data['id']) ? $data['id'] : 0,

				'username' 			=> isset($data['username']) ? $data['username'] : '',

				'profile_picture' 	=> isset($data['profile_pic_url']) ? $data['profile_pic_url'] : '',

				'full_name' 		=> isset($data['full_name']) ? $data['full_name'] : '',
			);
			$data_new['counts'] = array(
				'media' 			=> isset($data['edge_owner_to_timeline_media']['count'])
					? $data['edge_owner_to_timeline_media']['count'] : '',

				'followed_by' 		=> isset($data['edge_followed_by']['count']) ? $data['edge_followed_by']['count'] : '',

				'follows' 			=> isset($data['edge_follow']['count']) ? $data['edge_follow']['count'] : '',
			);


			$is_valid = ! empty($data_new['id']) ? true : false;

			if ( ! $is_valid ) {
				return $this->_set_response_error( $ret, 'data formatted - invalid user data!' );
			}
			return $this->_set_response_success( $ret, 'data formatted ok!', $data_new );
		}

		private function _get_user_box_visual( $data ) {

			$boxdata = array(
				'userid' 			=> $data['id'],
				'username' 			=> $data['username'],
				'user full name' 	=> $data['full_name'],
				'followed by' 		=> $data['counts']['followed_by'],
				'follows' 			=> $data['counts']['follows'],
			);

			ob_start();
		?>
			<ul class="method_get_user">
				<li>
					<img src="<?php echo esc_url( $data['profile_picture'] ); ?>" />
				</li>

				<?php
				$html = array();
				foreach ($boxdata as $key => $val) {
					$html[] = '<li>';
					$html[] = 	"<div>$key:</div>";
					$html[] = 	"<div>$val</div>";
					$html[] = '</li>';
				}
				$html = implode( PHP_EOL, $html );
				echo wp_kses($html, array(
					'li' => array(),
					'div' => array()
				));
				?>
			</ul>
		<?php
			return ob_get_clean();
		}


		//=========================================================================================
		// get_user_media
		public function get_user_media( $username, $params=array() ) {

			$params = array_replace_recursive( array(
				'use_cache' 	=> true,
			), $params );
			extract( $params );

			$api_method = 'get_user_media';
			$api_node = $this->_get_api_node( $api_method, $username );
			$cache_type = 'get_user_media';
			$cache_name = "user@{$username}_media";

			return $this->api_generic_method( $username, array(
				'use_cache' 	=> $use_cache,
				'api_method' 	=> $api_method,
				'api_node' 		=> $api_node,
				'cache_type' 	=> $cache_type,
				'cache_name' 	=> $cache_name,
			));
		}

		private function _get_user_media_data_parsed( $content, $params=array() ) {

			return $this->api_get_media_data_parsed( $content, $params );
		}

		private function _get_user_media_data_formatted( $data, $params=array() ) {

			return $this->api_get_media_data_formatted( $data, $params );
		}

		private function _get_user_media_box_visual( $data ) {

			return $this->api_get_media_box_visual( $data );
		}


		//=========================================================================================
		// get_tag_media
		public function get_tag_media( $hashtag, $params=array() ) {

			$params = array_replace_recursive( array(
				'use_cache' 	=> true,
			), $params );
			extract( $params );

			$api_method = 'get_tag_media';
			$api_node = $this->_get_api_node( $api_method, $hashtag );
			$cache_type = 'get_tag_media';
			$cache_name = "tag#{$hashtag}_media";

			return $this->api_generic_method( $hashtag, array(
				'use_cache' 	=> $use_cache,
				'api_method' 	=> $api_method,
				'api_node' 		=> $api_node,
				'cache_type' 	=> $cache_type,
				'cache_name' 	=> $cache_name,
			));
		}

		private function _get_tag_media_data_parsed( $content, $params=array() ) {

			return $this->api_get_media_data_parsed( $content, $params );
		}

		private function _get_tag_media_data_formatted( $data, $params=array() ) {

			return $this->api_get_media_data_formatted( $data, $params );
		}

		private function _get_tag_media_box_visual( $data ) {

			return $this->api_get_media_box_visual( $data );
		}


		//=========================================================================================
		// get_location_media
		public function get_location_media( $location_id, $params=array() ) {

			$params = array_replace_recursive( array(
				'use_cache' 	=> true,
			), $params );
			extract( $params );

			$api_method = 'get_location_media';
			$api_node = $this->_get_api_node( $api_method, $location_id );
			$cache_type = 'get_location_media';
			$cache_name = "loc&{$location_id}_media";

			return $this->api_generic_method( $location_id, array(
				'use_cache' 	=> $use_cache,
				'api_method' 	=> $api_method,
				'api_node' 		=> $api_node,
				'cache_type' 	=> $cache_type,
				'cache_name' 	=> $cache_name,
			));
		}

		private function _get_location_media_data_parsed( $content, $params=array() ) {

			return $this->api_get_media_data_parsed( $content, $params );
		}

		private function _get_location_media_data_formatted( $data, $params=array() ) {

			return $this->api_get_media_data_formatted( $data, $params );
		}

		private function _get_location_media_box_visual( $data ) {

			return $this->api_get_media_box_visual( $data );
		}


		//=========================================================================================
		// MISC
		private function _string_to_array( $id, $sep=',' ) {

			if ( empty($id) ) {
				return array();
			}

			if ( ! is_array($id) ) {

				$id_ = explode( ',', $id );
				$id_ = array_map( 'trim', $id_ );
				$id_ = array_unique( $id_ );
				$id_ = array_filter( $id_ );
				$id = $id_;
			}
			return $id;
		}
	}
}