<?php
/*
 * Define class Instaberg_Settings
 * Make sure you skip down to the end of this file, as there are a few
 * lines of code that are very important.
 */
!defined('ABSPATH') and exit;

if (class_exists('Instaberg_Settings') != true) {

	class Instaberg_Settings
	{
		/*
		 * Some required plugin information
		 */
		const VERSION = '1.0';

		private $save_status = false;

		/*
		 * Required __construct() function that initalizes the Instaberg_Settings
		 */
		public function __construct()
		{
			$this->save_settings();
		}

		private function save_settings()
		{
			$form_saved = isset( $_REQUEST['Instaberg-form-saved'] ) ? (boolean) $_REQUEST['Instaberg-form-saved'] : false;
			$box_id = isset( $_REQUEST['box_id'] ) ? (string) $_REQUEST['box_id'] : '';
			if( $form_saved === true && trim($box_id) != "" ){
				$params = isset( $_REQUEST[$box_id] ) ? $_REQUEST[$box_id] : array();
				if( count($params) > 0 ){
					update_option( $box_id, $params );
					$this->save_status = true;
				}
			}
		}

		/*
		 * Build options, method
		 * ---------------------
		 * this will create you interface via options array elements
		 */
		public function build_options($options = array(), $box_id = '')
		{
			// reset as array, this will stock all the html content, and at the end return it
			$html = array();

			if (count($options) == 0) {
				return 'Please fill with some options content first!';
			}

			$noRowElements = array(
				'message',
				'app',
				'box-style',
				'box_proxy',
				'box_apitesting',
				'box_media_max',
			);
			$settings      = array();

			// get the values from DB
			$box_id = $box_id;

			$settings = get_option( $box_id );

			$html[] = '<form method="POST" class="Instaberg-form" id="' . ($box_id) . '" action="">';

			$html[] = '<div style="margin-bottom: 20px;" class="Instaberg-message Instaberg-success ' . ( $this->save_status == false ? 'Instaberg-display-none' : '' ) . '">' . esc_html__('Saved successfully!', 'instaberg') . '</div>';
			// create a hidden input for sending the prefix
			$html[] = 	'<input type="hidden" name="box_id" value="' . ($box_id) . '" />';
			$html[] = 	'<input type="hidden" name="Instaberg-form-saved" value="1" />';

			$box = isset($options) ? $options : array();
			$html[] = $this->tabs_header($box); // tabs html header

			// loop the box elements
			if (isset($options['elements']) && count($options['elements']) > 0) {

				// loop the box elements now
				foreach ($options['elements'] as $elm_id => $value) {

					// some helpers. Reset an each loop, prevent collision
					$val          = '';
					$select_value = '';
					$checked      = '';
					$option_name  = isset($option_name) ? $option_name : '';

					// Set default value to $val
					if (isset($value['std'])) {
						$val = $value['std'];
					}

					// If the option is already saved, ovveride $val
					if ( ( $value['type'] != 'info' ) ) {
						if ( isset($settings[($elm_id)] )
							&& (
								( !is_array($settings[($elm_id)]) && @trim($settings[($elm_id)]) != "" )
								||
								( is_array($settings[($elm_id)]) )
							)
						) {
								$val = $settings[( $elm_id )];

								// Striping slashes of non-array options
								if ( !is_array($val) ) {
									$val = stripslashes( $val );
								}
						}
					}

					// If there is a description save it for labels
					$explain_value = '';
					if (isset($value['desc'])) {
						$explain_value = $value['desc'];
					}

					if (!in_array($value['type'], $noRowElements)) {
						// the row and the label
						$html[] = '<div class="Instaberg-form-row' . ($this->tabs_elements($box, $elm_id)) . '">
							   <label for="' . ($elm_id) . '">' . (isset($value['title']) ? $value['title'] : '') . '</label>
							   <div class="Instaberg-form-item">';
					}

					switch ($value['type']) {

						// Basic text input
						case 'text':
							$html[] = '<input ' . (isset($value['force_width']) ? "style='width:" . ($value['force_width']) . "px;'" : '') . ' id="' . esc_attr($elm_id) . '" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . ']" type="text" value="' . esc_attr($val) . '" />';
							break;


						 case 'list_of_colors':
							$html[] = '<div class="Instaberg-list-of-colors">';
							$html[] =   '<a href="#" class="button button-primary button-large Instaberg-add-color">' . esc_html__('Add new color', 'instaberg') . '</a>';
							$html[] =   '<ul>';

							if( count(Instaberg()->the_colors()) > 0 ){
								foreach ( Instaberg()->the_colors() as $color_name => $rgb ) {

									$html[] = '<li>';
									$html[] =   '<a href="#" class="Instaberg-lc-remove-color"><i class="dashicons dashicons-no-alt"></i></a>';
									$html[] =   '<input type="text" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . '][name][]" class="Instaberg-lc-color-name" placeholder="' . esc_attr__('Color Name', 'instaberg') . '" value="' . ( $color_name ) . '" />';
									$html[] =   '<input type="text" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . '][color][]" class="Instaberg-lc-color-hex Instaberg-wp-color-picker" value="' . ( Instaberg()->rgb2html( $rgb[0], $rgb[1], $rgb[2] ) ) . '" />';
									$html[] = '</li>';
								}
							}

							$html[] =   '</ul>';
							$html[] =   '';
							$html[] = '</div>';
							break;

						case 'box-style':

							$html[] = '<div class="Instaberg-form-row Instaberg_template">';

							$html[] = '<div class="Instaberg-box-style-title">';
							$html[] =   '<h2>' . ( $value['title'] ) . '</h2>';
							$html[] =   '<p>' . ( $value['box_desc'] ) . ' </p>';
							$html[] = '</div>';
							$html[] = '<div class="Instaberg-box-style">';
							$html[] =   '<div class="Instaberg-box-preview"><span></span></div>';
							$html[] =   '<div class="Instaberg-box-settings">';
							$html[] =       '<a href="#" class="Instaberg-box-settings-title on" data-rel="border-radius">' . esc_html__('Border Radius', 'instaberg') . '</a>';
							$html[] =       '<a href="#" class="Instaberg-box-settings-title" data-rel="box-shadow">' . esc_html__('Box Shadow', 'instaberg') . '</a>';
							$html[] =       '<a href="#" class="Instaberg-box-settings-title" data-rel="box-spacing">' . esc_html__('Box Spacing', 'instaberg') . '</a>';
							$html[] =       '<div class="Instaberg-box-content" data-rel="box-spacing">';
							$html[] =           '<fieldset>';
							$html[] =               '<label>';
							$html[] =                   '<span>' . esc_html__('Box Margin', 'instaberg') . '</span>';
							$html[] =                   '<div class="Instaberg-box-spacing">';
							$html[] =                       '<div class="Instaberg-box-spacing-margin">';
							$html[] =                           '<span class="Instaberg-box-spacing-margin-top Instaberg-box-spacing-input">';
							$html[] =                               '<input type="text" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . '][margin][top]" value="' . ( isset($val['margin']['top']) ? $val['margin']['top'] : '0px') . '">';
							$html[] =                           '</span>';
							$html[] =                           '<span class="Instaberg-box-spacing-margin-right Instaberg-box-spacing-input">';
							$html[] =                               '<input type="text" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . '][margin][right]" value="' . ( isset($val['margin']['right']) ? $val['margin']['right'] : '0px') . '">';
							$html[] =                           '</span>';
							$html[] =                           '<span class="Instaberg-box-spacing-margin-bottom Instaberg-box-spacing-input">';
							$html[] =                               '<input type="text" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . '][margin][bottom]" value="' . ( isset($val['margin']['bottom']) ? $val['margin']['bottom'] : '0px') . '">';
							$html[] =                           '</span>';
							$html[] =                           '<span class="Instaberg-box-spacing-margin-left Instaberg-box-spacing-input">';
							$html[] =                               '<input type="text" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . '][margin][left]" value="' . ( isset($val['margin']['left']) ? $val['margin']['left'] : '0px') . '">';
							$html[] =                           '</span>';
							$html[] =                           '<span class="Instaberg-box-spacing-content">' . esc_html__('content', 'instaberg') . '</span>';
							$html[] =                       '</div>';
							$html[] =                   '</div>';
							$html[] =               '</label>';
							$html[] =               '<label>';
							$html[] =                   '<span>' . esc_html__('Box Padding', 'instaberg') . '</span>';
							$html[] =                   '<div class="Instaberg-box-spacing">';
							$html[] =                       '<div class="Instaberg-box-spacing-margin">';
							$html[] =                           '<span class="Instaberg-box-spacing-margin-top Instaberg-box-spacing-input">';
							$html[] =                               '<input type="text" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . '][padding][top]" value="' . ( isset($val['padding']['top']) ? $val['padding']['top'] : '0px') . '">';
							$html[] =                           '</span>';
							$html[] =                           '<span class="Instaberg-box-spacing-margin-right Instaberg-box-spacing-input">';
							$html[] =                               '<input type="text" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . '][padding][right]" value="' . ( isset($val['padding']['right']) ? $val['padding']['right'] : '0px') . '">';
							$html[] =                           '</span>';
							$html[] =                           '<span class="Instaberg-box-spacing-margin-bottom Instaberg-box-spacing-input">';
							$html[] =                               '<input type="text" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . '][padding][bottom]" value="' . ( isset($val['padding']['bottom']) ? $val['padding']['bottom'] : '0px') . '">';
							$html[] =                           '</span>';
							$html[] =                           '<span class="Instaberg-box-spacing-margin-left Instaberg-box-spacing-input">';
							$html[] =                               '<input type="text" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . '][padding][left]" value="' . ( isset($val['padding']['left']) ? $val['padding']['left'] : '0px') . '">';
							$html[] =                           '</span>';
							$html[] =                           '<span class="Instaberg-box-spacing-content">' . esc_html__('content', 'instaberg') . '</span>';
							$html[] =                       '</div>';
							$html[] =                   '</div>';
							$html[] =               '</label>';
							$html[] =           '</fieldset>';
							$html[] =       '</div>';
							$html[] =       '<div class="Instaberg-box-content" data-rel="border-radius" style="display: block">';
							$html[] =           '<fieldset>';
							$html[] =               '<label>';
							$html[] =                   '<span>' . esc_html__('All Corners', 'instaberg') . '</span>';
							$html[] =                   '<em>px</em><input type="text" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . '][radius][all]" value="' . ( isset($val['radius']['all']) ? $val['radius']['all'] : '10') . '"/>';
							$html[] =                   '<div class="Instaberg-box-settings-slider" data-min="0" data-max="200" data-def="' . ( isset($val['radius']['all']) ? $val['radius']['all'] : '10') . '"></div>';
							$html[] =               '</label>';
							$html[] =               '<label>';
							$html[] =                   '<span>' . esc_html__('Top Left', 'instaberg') . '</span>';
							$html[] =                   '<em>px</em><input type="text" value="' . ( isset($val['radius']['top-left']) ? $val['radius']['top-left'] : '10') . '" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . '][radius][top-left]"/>';
							$html[] =                   '<div class="Instaberg-box-settings-slider" data-min="0" data-max="200" data-def="' . ( isset($val['radius']['top-left']) ? $val['radius']['top-left'] : '10') . '"></div>';
							$html[] =               '</label>';
							$html[] =               '<label>';
							$html[] =                   '<span>' . esc_html__('Top Right', 'instaberg') . '</span>';
							$html[] =                   '<em>px</em><input type="text" value="' . ( isset($val['radius']['top-right']) ? $val['radius']['top-right'] : '10') . '" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . '][radius][top-right]"/>';
							$html[] =                   '<div class="Instaberg-box-settings-slider" data-min="0" data-max="200" data-def="' . ( isset($val['radius']['top-right']) ? $val['radius']['top-right'] : '10') . '"></div>';
							$html[] =               '</label>';
							$html[] =               '<label>';
							$html[] =                   '<span>' . esc_html__('Bottom Left', 'instaberg') . '</span>';
							$html[] =                   '<em>px</em><input type="text" value="' . ( isset($val['radius']['bottom-left']) ? $val['radius']['bottom-left'] : '10') . '" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . '][radius][bottom-left]"/>';
							$html[] =                   '<div class="Instaberg-box-settings-slider" data-min="0" data-max="200" data-def="' . ( isset($val['radius']['bottom-left']) ? $val['radius']['bottom-left'] : '10') . '"></div>';
							$html[] =               '</label>';
							$html[] =               '<label>';
							$html[] =                   '<span>' . esc_html__('Bottom Right', 'instaberg') . '</span>';
							$html[] =                   '<em>px</em><input type="text" value="' . ( isset($val['radius']['bottom-right']) ? $val['radius']['bottom-right'] : '10') . '" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . '][radius][bottom-right]"/>';
							$html[] =                   '<div class="Instaberg-box-settings-slider" data-min="0" data-max="200" data-def="' . ( isset($val['radius']['bottom-right']) ? $val['radius']['bottom-right'] : '10') . '"></div>';
							$html[] =               '</label>';

							$html[] =               '<label style="margin-top: 30px">';
							$html[] =                   '<span>' . esc_html__('Border Width', 'instaberg') . '</span>';
							$html[] =                   '<em>px</em><input type="text" value="' . ( isset($val['radius']['border-width']) ? $val['radius']['border-width'] : '0') . '" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . '][radius][border-width]" />';
							$html[] =                   '<div class="Instaberg-box-settings-slider" data-min="0" data-max="100" data-def="' . ( isset($val['radius']['border-width']) ? $val['radius']['border-width'] : '0') . '"></div>';
							$html[] =               '</label>';
							$html[] =               '<label class="Instaberg-box-settings-one-row">';
							$html[] =                   '<span>' . esc_html__('Border Style', 'instaberg') . '</span>';
							$html[] =                   '<select name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . '][radius][border-style]">';
							$html[] =                       '<option value="solid" ' . ( isset($val['radius']['border-style']) && $val['radius']['border-style'] == 'solid' ? 'selected' : '' ) .'>solid</option>';
							$html[] =                       '<option value="dotted" ' . ( isset($val['radius']['border-style']) && $val['radius']['border-style'] == 'dotted' ? 'selected' : '' ) .'>dotted</option>';
							$html[] =                       '<option value="dashed" ' . ( isset($val['radius']['border-style']) && $val['radius']['border-style'] == 'dashed' ? 'selected' : '' ) .'>dashed</option>';
							$html[] =                       '<option value="double" ' . ( isset($val['radius']['border-style']) && $val['radius']['border-style'] == 'double' ? 'selected' : '' ) .'>double</option>';
							$html[] =                       '<option value="none" ' . ( isset($val['radius']['border-style']) && $val['radius']['border-style'] == 'none' ? 'selected' : '' ) .'>none</option>';
							$html[] =                   '</select>';
							$html[] =               '</label>';
							$html[] =               '<label class="Instaberg-box-settings-one-row">';
							$html[] =                   '<span>' . esc_html__('Border Color', 'instaberg') . '</span>';
							$html[] =                   '<input class="Instaberg-wp-color-picker-box" data-alpha="true" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . '][radius][border-color]" type="text" value="' . ( isset($val['radius']['border-color']) ? $val['radius']['border-color'] : '#fee50e') . '" />';
							$html[] =               '</label>';
							$html[] =               '<label class="Instaberg-box-settings-one-row">';
							$html[] =                   '<span>' . esc_html__('Box Background', 'instaberg') . '</span>';
							$html[] =                   '<input class="Instaberg-wp-color-picker-box" data-alpha="true" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . '][radius][background-color]" type="text" value="' . ( isset($val['radius']['background-color']) ? $val['radius']['background-color'] : '#5f3a71') . '" />';
							$html[] =               '</label>';
							$html[] =           '</fieldset>';
							$html[] =       '</div>';
							$html[] =       '<div class="Instaberg-box-content" data-rel="box-shadow">';
							$html[] =           '<fieldset>';
							$html[] =               '<label>';
							$html[] =                   '<span>' . esc_html__('Horizontal Length', 'instaberg') . '</span>';
							$html[] =                   '<em>px</em><input type="text" value="' . ( isset($val['box-shadow']['horizontal']) ? $val['box-shadow']['horizontal'] : '10') . '" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . '][box-shadow][horizontal]"/>';
							$html[] =                   '<div class="Instaberg-box-settings-slider" data-min="-200" data-max="200" data-def="' . ( isset($val['box-shadow']['horizontal']) ? $val['box-shadow']['horizontal'] : '10') . '"></div>';
							$html[] =               '</label>';
							$html[] =               '<label>';
							$html[] =                   '<span>' . esc_html__('Vertical Length', 'instaberg') . '</span>';
							$html[] =                   '<em>px</em><input type="text" value="' . ( isset($val['box-shadow']['vertical']) ? $val['box-shadow']['vertical'] : '10') . '" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . '][box-shadow][vertical]"/>';
							$html[] =                   '<div class="Instaberg-box-settings-slider" data-min="-200" data-max="200" data-def="' . ( isset($val['box-shadow']['vertical']) ? $val['box-shadow']['vertical'] : '10') . '"></div>';
							$html[] =               '</label>';
							$html[] =               '<label>';
							$html[] =                   '<span>' . esc_html__('Blur Radius', 'instaberg') . '</span>';
							$html[] =                   '<em>px</em><input type="text" value="' . ( isset($val['box-shadow']['blur']) ? $val['box-shadow']['blur'] : '10') . '" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . '][box-shadow][blur]"/>';
							$html[] =                   '<div class="Instaberg-box-settings-slider" data-min="0" data-max="200" data-def="' . ( isset($val['box-shadow']['blur']) ? $val['box-shadow']['blur'] : '10') . '"></div>';
							$html[] =               '</label>';
							$html[] =               '<label>';
							$html[] =                   '<span>' . esc_html__('Spread Radius', 'instaberg') . '</span>';
							$html[] =                   '<em>px</em><input type="text" value="' . ( isset($val['box-shadow']['spread']) ? $val['box-shadow']['spread'] : '0') . '" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . '][box-shadow][spread]"/>';
							$html[] =                   '<div class="Instaberg-box-settings-slider" data-min="-200" data-max="200" data-def="' . ( isset($val['box-shadow']['spread']) ? $val['box-shadow']['spread'] : '0') . '"></div>';
							$html[] =               '</label>';
							$html[] =               '<label class="Instaberg-box-settings-one-row">';
							$html[] =                   '<span>' . esc_html__('Shadow Color', 'instaberg') . '</span>';
							$html[] =                   '<input class="Instaberg-wp-color-picker-box" data-alpha="true" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . '][box-shadow][shadow]" type="text" value="' . ( isset($val['box-shadow']['spread']) ? $val['box-shadow']['shadow'] : '#23282d') . '" />';
							$html[] =               '</label>';
							$html[] =              '<label>';
							$html[] =                   '<span>' . esc_html__('Opacity', 'instaberg') . '</span>';
							$html[] =                   '<em>%</em><input type="text" value="' . ( isset($val['box-shadow']['opacity']) ? $val['box-shadow']['opacity'] : '0') . '" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . '][box-shadow][opacity]"/>';
							$html[] =                   '<div class="Instaberg-box-settings-slider" data-min="0" data-max="100" data-def="' . ( isset($val['box-shadow']['opacity']) ? $val['box-shadow']['opacity'] : '0') . '" data-step="1"></div>';
							$html[] =               '</label>';
							$html[] =               '<label class="Instaberg-box-settings-one-row">';
							$html[] =                   '<span>' . esc_html__('Box Shadow type', 'instaberg') . '</span>';
							$html[] =                   '<select name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . '][box-shadow][type]">';
							$html[] =                       '<option value="outline" ' . ( isset($val['box-shadow']['type']) && $val['box-shadow']['type'] == 'outline' ? 'selected' : '' ) .'>Outline</option>';
							$html[] =                       '<option value="inset" ' . ( isset($val['box-shadow']['type']) && $val['box-shadow']['type'] == 'inset' ? 'selected' : '' ) .'>Inset</option>';
							$html[] =                   '</select>';
							$html[] =               '</label>';
							$html[] =           '</fieldset>';
							$html[] =       '</div>';
							$html[] =   '</div>';
							$html[] = '<textarea class="Instaberg-box-settings-export-style" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . '][output]"></textarea>';

							$html[] = '</div>';
							$html[] = '</div>';
							break;

						// Basic text input
						case 'timepicker':
							$html[] = '<input ' . (isset($value['force_width']) ? "style='width:" . ($value['force_width']) . "px;'" : '') . ' id="' . esc_attr($elm_id) . '" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . ']" type="text" value="' . esc_attr($val) . '" class="' . ( Instaberg()->alias . '-timepicker') . '" />';
							break;

						// Basic text input
						case 'password_access':
							$html[] = '<input id="' . esc_attr($elm_id) . '" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . ']" type="text" value="' . esc_attr($val) . '" class="' . ( Instaberg()->alias . '-password') . '" />';
							$html[] = '<a href="#upload" target="blank" class="' . ( Instaberg()->alias ) . '-button ' . ( Instaberg()->alias ) . '-small red ' . ( Instaberg()->alias ) . '-btn-new-password">' . esc_html__('Generate new password', 'instaberg') . '</a>';
							break;

						// Wordpress color picker input
						case 'color_picker':
							$html[] = '<input class="Instaberg-wp-color-picker" data-alpha="true" ' . ( isset($value['force_width']) ? "style='width:" . ( $value['force_width'] ) . "px;'" : '' ) . ' id="' . esc_attr( $elm_id ) . '" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . ']" type="text" value="' . esc_attr( $val ) . '" />';
						break;

						case 'template_typography':
							if( $value['options'] ){
								$html[] = '<div class="Instaberg-typo-wrapper">';
								$google_fonts = Instaberg()->getAllGfonts();

								if( !isset($val) || !is_array($val) || count($val) == 0 ){
									$val = array();
								}
								foreach ($value['options'] as $option_key => $option_value) {

									// build some defaults if not saved
									if( !isset($val[$option_key]['family']) )              $val[$option_key]['family'] = $option_value['family'];
									if( !isset($val[$option_key]['size']) )                $val[$option_key]['size'] = $option_value['size'];
									if( !isset($val[$option_key]['weight']) )              $val[$option_key]['weight'] = isset($option_value['weight']) ? $option_value['weight'] : '';
									if( !isset($val[$option_key]['color']) )               $val[$option_key]['color'] = $option_value['color'];
									if( !isset($val[$option_key]['background-color']) )    $val[$option_key]['background-color'] = isset($option_value['background-color']) ? $option_value['background-color'] : '';

									$html[] =   '<fieldset class="Instaberg-typo-row">';
									$html[] =       '<legend class="Instaberg-typo-title">' . ( $option_value['title'] ) . '</legend>';
									$html[] =       '<div class="Instaberg-typo-font-preview">The quick brown fox jumps over the lazy dog</div>';
									$html[] =       '<div class="Instaberg-typo-font-family">';
									$html[] =           '<span>' . esc_html__( 'Font-family', 'instaberg') . ':</span>';
									$html[] =           '<select name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . '][' . ( $option_key ) . '][family]">';

									if( count($google_fonts) > 0 ){
										foreach ($google_fonts as $font_key => $font ) {
											$html[] = '<option value="' . ( $font_key ) . '" ' . ( isset($val[$option_key]['family']) && $val[$option_key]['family'] == $font ? 'selected' : '' ) . '>' . ( $font ) . '</option>';
										}
									}
									$html[] =           '</select>';
									$html[] =       '</div>';

									$html[] =       '<div class="Instaberg-typo-font-size">';
									$html[] =           '<span>' . esc_html__( 'Font-size', 'instaberg') . ':</span>';
									$html[] =           '<select name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . '][' . ( $option_key ) . '][size]">';
									foreach ( range( 10, 34 ) as $size ) {
										$html[] =           '<option value="' . ( $size ) . '" ' . ( isset($val[$option_key]['size']) && $val[$option_key]['size'] == $size ? 'selected' : '' ) .'>' . ( $size ) . 'px</option>';
									}
									$html[] =            '</select>';
									$html[] =        '</div>';

									$html[] =       '<div class="Instaberg-typo-font-weight">';
									$html[] =           '<span>' . esc_html__( 'Font-weight', 'instaberg') . ':</span>';
									$html[] =           '<select name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . '][' . ( $option_key ) . '][weight]">';
									foreach ( array('normal', 'bold') as $weight ) {
										$html[] =           '<option value="' . ( $weight ) . '" ' . ( isset($val[$option_key]['weight']) && $val[$option_key]['weight'] == $weight ? 'selected' : '' ) .'>' . ( $weight ) . '</option>';
									}
									$html[] =            '</select>';
									$html[] =        '</div>';

									$html[] =       '<div class="Instaberg-typo-font-color">';
									$html[] =           '<span>' . esc_html__( 'Font-color', 'instaberg') . ':</span>';
									$html[] =           '<input class="Instaberg-wp-color-picker-typo" data-alpha="true" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . '][' . ( $option_key ) . '][color]" type="text" value="' . ( isset($val[$option_key]['color']) ? $val[$option_key]['color'] : '#000') . '" />';
									$html[] =           '<div class="Instaberg-wp-color-picker-wrapper"></div>';
									$html[] =        '</div>';


									if( $val[$option_key]['background-color'] != "" ){
										$html[] =       '<div class="Instaberg-typo-background-color">';
										$html[] =           '<span>' . esc_html__( 'Background-color', 'instaberg') . ':</span>';
										$html[] =           '<input class="Instaberg-wp-color-picker-typo" data-alpha="true" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . '][' . ( $option_key ) . '][background-color]" type="text" value="' . ( isset($val[$option_key]['background-color']) ? $val[$option_key]['background-color'] : '#000') . '" />';
										$html[] =           '<div class="Instaberg-wp-color-picker-wrapper"></div>';
										$html[] =        '</div>';
									}



									$html[] =   '</fieldset>';
								}
								$html[] = '</div>';
							}
						break;

						// Basic checkbox input
						case 'checkbox':
							$html[] = '<input ' . (isset($value['force_width']) ? "style='width:" . ($value['force_width']) . "px;'" : '') . ' ' . ($val == true ? 'checked' : '') . ' id="' . esc_attr($elm_id) . '" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . ']" type="checkbox" value="" />';
							break;

						// Basic checkbox input
						case 'image_radio':

							if( isset($value['options']) && count($value['options']) > 0 ){
								$html[] = '<div class="Instaberg-select-wrapper">';
									foreach ($value['options'] as $option_key => $option_value ) {
										$html[] =   '<label ' . (isset($value['force_width']) ? "style='width:" . ($value['force_width']) . "px;'" : '') . ' class="Instaberg-select-item ' . ($val == $option_key ? 'select-on' : '') . '">';
										$html[] =       '<span>' . ( $option_value['caption'] ) . '</span>';
										$html[] =       '<img src="' . ( $option_value['img'] ) . '" />';
										$html[] =       '<input name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . ']" ' . ($val == $option_key ? 'checked' : '') . ' type="radio" value="' . ( $option_key ) . '" />';
										$html[] =   '</label>';
									}
								$html[] = '</div>';
							}
							break;

						// Basic checkbox input
						case 'html_radio':

							if( isset($value['options']) && count($value['options']) > 0 ){
								$html[] = '<div class="Instaberg-select-wrapper">';
									foreach ($value['options'] as $option_key => $option_value ) {
										$html[] =   '<label ' . (isset($value['force_width']) ? "style='width:" . ($value['force_width']) . "px;'" : '') . ' class="Instaberg-select-item ' . ($val == $option_key ? 'select-on' : '') . '">';
										$html[] =       '<span>' . ( $option_value['caption'] ) . '</span>';
										$html[] =       '<div>' . ( $option_value['html'] ) . '</div>';
										$html[] =       '<input name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . ']" ' . ($val == $option_key ? 'checked' : '') . ' type="radio" value="' . ( $option_key ) . '" />';
										$html[] =   '</label>';
									}
								$html[] = '</div>';
							}
							break;

						// Basic upload_image
						case 'upload_image':

							$html[] = '<div class="' . ( Instaberg()->alias ) . '-upload-box-wrapper">';
							$html[] =   '<div class="' . ( Instaberg()->alias ) . '-upload-image">';
							if( (int)$val > 0 ){
								$html[] = wp_get_attachment_image( $val, array( '75', '75' ) );
							}
							$html[] =   '</div>';
							$html[] =   '<input type="hidden" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . ']" value="' . ( (int)$val ) . '" />';
							$html[] =   '<a href="#upload" target="blank" class="' . ( Instaberg()->alias ) . '-button ' . ( Instaberg()->alias ) . '-small yellow ' . ( Instaberg()->alias ) . '-btn-upload">' . esc_html__('Upload Image', 'instaberg') . '</a>';
							$html[] =   '<a href="#upload" target="blank" class="' . ( Instaberg()->alias ) . '-button ' . ( Instaberg()->alias ) . '-small red ' . ( Instaberg()->alias ) . '-btn-upload-delete">' . esc_html__('Delete Image', 'instaberg') . '</a>';
							$html[] = '</div>';

							break;

						case 'import':
							$html[] =   '<a href="#" target="blank" data-id="' . ( Instaberg()->get_campaign_id() ) . '" class="' . ( Instaberg()->alias ) . '-button yellow ' . ( Instaberg()->alias ) . '-import-now">' . esc_html__('Import now', 'instaberg') . '</a>';

							break;

						case 'export':
							$html[] = '<div class="' . ( Instaberg()->alias ) . '-export-settings">';

							$html[] =   '<a href="#" target="blank" class="' . ( Instaberg()->alias ) . '-button yellow ' . ( Instaberg()->alias ) . '-export-now">' . esc_html__('Export now', 'instaberg') . '</a>';
							$html[] = '</div>';

							break;

						// Basic textarea
						case 'textarea':
							$cols = "120";
							if (isset($value['cols'])) {
								$cols = $value['cols'];
							}
							$height = "style='height:120px;'";
							if (isset($value['height'])) {
								$height = "style='height:{$value['height']};'";
							}

							$html[] = '<textarea id="' . esc_attr($elm_id) . '" ' . $height . ' cols="' . ($cols) . '" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . ']">' . esc_attr($val) . '</textarea>';
							break;

						// Basic html/text message
						case 'message':
							$html[] = '<div class="Instaberg-message Instaberg-' . ($value['status']) . ' ' . ($this->tabs_elements($box, $elm_id)) . '">' . ($value['html']) . '</div>';
							break;


						// Basic html/text message
						case 'html':
							$__html = str_replace("#box_id#", $box_id, $value['html']);
							$__html = str_replace("#option_name#", $box_id, $__html);
							$__html = str_replace("#option_id#", $elm_id, $__html);
							$__html = str_replace("#option_value#", $val, $__html);

							$html[] = $__html;
							break;

						// Select Box
						case 'select':
							$html[] = '<select ' . (isset($value['force_width']) ? "style='width:" . ($value['force_width']) . "px;'" : '') . ' name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . ']" id="' . esc_attr($elm_id) . '">';

							foreach ($value['options'] as $key => $option) {
								$selected = '';
								if ($val != '') {
									if ($val == $key) {
										$selected = ' selected="selected"';
									}
								}
								$html[] = '<option' . $selected . ' value="' . esc_attr($key) . '">' . esc_html($option) . '</option>';
							}
							$html[] = '</select>';
							break;

						// multiselect Box
						case 'multiselect':
							$html[] = '<select multiple="multiple" size="3" name="[' . ( $box_id ) . '][' . esc_attr( $elm_id ) . '][]" id="' . esc_attr($elm_id) . '">';

							if (count($option) > 1) {
								foreach ($value['options'] as $key => $option) {
									$selected = '';
									if ($val != '') {
										if (in_array($key, $val)) {
											$selected = ' selected="selected"';
										}
									}
									$html[] = '<option' . $selected . ' value="' . esc_attr($key) . '">' . esc_html($option) . '</option>';
								}
							}
							$html[] = '</select>';
							break;

						// Select Box
						case 'multi_select_box':
							if( count($value['subfields']) > 0 ){
								foreach ($value['subfields'] as $kk => $vv) {

									$muval = '';
									// Set default value to $val
									if ( isset( $vv['std']) ) {
										$muval = $vv['std'];
									}

									$html[] = '<span>' . ( $vv['title'] ) . ':</span>';
									$html[] = '<select ' . ( isset($vv['force_width']) ? "style='width:" . ( $vv['force_width'] ) . "px;'" : '' ) . ' name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . '][' . ( $kk ) . ']" id="' . esc_attr( $elm_id ) . '[' . ( $kk ) . ']">';

									foreach ($vv['options'] as $key2 => $option ) {
										$selected = '';
										if( isset($val[$kk]) && $val[$kk] != '' ) {
											if ( $val[$kk] == $key2 ) { $selected = ' selected="selected"';}
										}
										else if ( $muval != '' ) {
											if ( $muval == $key2 ) { $selected = ' selected="selected"';}
										}

										$html[] = '<option'. $selected .' value="' . esc_attr( $key2 ) . '">' . esc_html( $option ) . '</option>';
									 }
									$html[] = '</select>&nbsp;&nbsp;';
								}
							}

						break;

						// multiselect Box
						case 'multiselect_left2right':
							$available = array(); $selected = array();
							foreach ($value['options'] as $key => $option ) {
								if( $val != '' ) {
									if ( in_array($key, $val) ) { $selected[] = $key; }
								}
							}
							$available = array_diff(array_keys($value['options']), $selected);

							$html[] = '<div class="Instaberg-multiselect-half Instaberg-multiselect-available" style="margin-right: 2%;">';
							if( isset($value['info']['left']) ){
								$html[] = '<h5>' . ( $value['info']['left'] ) . '</h5>';
							}
							$html[] = '<select multiple="multiple" size="' . (isset($value['rows_visible']) ? $value['rows_visible'] : 5) . '" name="[' . ( $box_id ) . '][' . esc_attr( $elm_id ) . '-available][]" id="' . esc_attr( $elm_id ) . '-available" class="multisel_l2r_available">';

							if(count($available) > 0){
								foreach ($value['options'] as $key => $option ) {
									if ( !in_array($key, $available) ) continue 1;
									$html[] = '<option value="' . esc_attr( $key ) . '">' . esc_html( $option ) . '</option>';
								}
							}
							$html[] = '</select>';

							$html[] = '</div>';

							$html[] = '<div class="Instaberg-multiselect-half Instaberg-multiselect-selected">';
							if( isset($value['info']['right']) ){
								$html[] = '<h5>' . ( $value['info']['right'] ) . '</h5>';
							}
							$html[] = '<select multiple="multiple" size="' . (isset($value['rows_visible']) ? $value['rows_visible'] : 5) . '" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . '][]" id="' . esc_attr( $elm_id ) . '" class="multisel_l2r_selected">';

							if(count($selected) > 0){
								foreach ($value['options'] as $key => $option ) {
									if ( !in_array($key, $selected) ) continue 1;
									$isselected = ' selected="selected"';
									$html[] = '<option'. $isselected .' value="' . esc_attr( $key ) . '">' . esc_html( $option ) . '</option>';
								}
							}
							$html[] = '</select>';
							$html[] = '</div>';
							$html[] = '<div style="clear:both"></div>';
							$html[] = '<div class="multisel_l2r_btn" style="">';
							$html[] = '<span style="display: inline-block; text-align: center;"><input id="' . esc_attr( $elm_id ) . '-moveright" type="button" value="Move Right" class="moveright Instaberg-button Instaberg-small yellow"></span>';
							$html[] = '<span style="display: inline-block; text-align: center;"><input id="' . esc_attr( $elm_id ) . '-moverightall" type="button" value="Move Right All" class="moverightall Instaberg-small Instaberg-button yellow"></span>';
							$html[] = '<span style="display: inline-block; margin-left: 60px; text-align: left;"><input id="' . esc_attr( $elm_id ) . '-moveleft" type="button" value="Move Left" class="moveleft Instaberg-button Instaberg-small yellow"></span>';
							$html[] = '<span style="display: inline-block; text-align: center;"><input id="' . esc_attr( $elm_id ) . '-moveleftall" type="button" value="Move Left All" class="moveleftall Instaberg-small Instaberg-button yellow"></span>';
							$html[] = '</div>';
						break;

						// Envato Login box
						case 'instagram-login':
							if ( function_exists('Instaberg_Instagram_Helper') ) {
								if ( is_object( Instaberg_Instagram_Helper() ) ) {
									$html[] = Instaberg_Instagram_Helper()->auth( $settings );
								}
							}
							break;

						case 'box_media_max':

							$proxy_info = array(
								'media_max_user' => isset($settings['media_max_user']) ? $settings['media_max_user'] : 75,
								'media_max_tag' => isset($settings['media_max_tag']) ? $settings['media_max_tag'] : 100,
								'media_max_location' => isset($settings['media_max_location']) ? $settings['media_max_location'] : 125,
							);

							// Striping slashes of non-array options
							foreach ($proxy_info as $proxy_kk => $proxy_vv) {
								$proxy_info["$proxy_kk"] = stripslashes( $proxy_vv );
							}

							$opt_range = InstabergUtils()->doRange( range(10, 250, 5) );

							ob_start();
							?>

<div class="Instaberg-form-row Instaberg_config Instaberg_box_proxy Instaberg_box_media_max">

	<label for="Instaberg_box_proxy_content"><?php esc_html_e('Max Media', 'instaberg'); ?></label>

	<div class="Instaberg-form-item Instaberg_box_proxy_content">

		<div>
			<select id="media_max_user" name="Instaberg_Plugin_Settings[media_max_user]" class="medium">
				<?php
				foreach ( $opt_range as $key => $val ) {
					$is_selected = $key == $proxy_info['media_max_user'] ? ' selected="true"' : '';
					echo '<option value="' . $key . '"' . $is_selected . '>' . $val . '</option>';
				}
				?>
			</select>
			<span class="Instaberg-form-note"><?php esc_html_e('user profile max media', 'instaberg'); ?></span>
		</div>

		<div>
			<select id="media_max_tag" name="Instaberg_Plugin_Settings[media_max_tag]" class="medium">
				<?php
				foreach ( $opt_range as $key => $val ) {
					$is_selected = $key == $proxy_info['media_max_tag'] ? ' selected="true"' : '';
					echo '<option value="' . $key . '"' . $is_selected . '>' . $val . '</option>';
				}
				?>
			</select>
			<span class="Instaberg-form-note"><?php esc_html_e('hastag max media', 'instaberg'); ?></span>
		</div>

	</div>

</div>

							<?php
							$html[] = ob_get_clean();
							break;

						case 'box_proxy':

							$proxy_info = array(
								'proxy_server' => isset($settings['proxy_server']) ? $settings['proxy_server'] : '',
								'proxy_username' => isset($settings['proxy_username']) ? $settings['proxy_username'] : '',
								'proxy_password' => isset($settings['proxy_password']) ? $settings['proxy_password'] : '',
							);

							// Striping slashes of non-array options
							foreach ($proxy_info as $proxy_kk => $proxy_vv) {
								$proxy_info["$proxy_kk"] = stripslashes( $proxy_vv );
							}

							ob_start();
							?>

<div class="Instaberg-form-row Instaberg_config Instaberg_box_proxy">

	<label for="Instaberg_box_proxy_content"><?php esc_html_e('Proxy', 'instaberg'); ?></label>

	<div class="Instaberg-form-item Instaberg_box_proxy_content">

		<div>
			<input id="proxy_server" name="Instaberg_Plugin_Settings[proxy_server]" type="text" value="<?php echo esc_attr($proxy_info['proxy_server']); ?>" placeholder="server ip">
			<span class="Instaberg-form-note"><?php esc_html_e('the server ip', 'instaberg'); ?></span>
		</div>

		<div>
			<input id="proxy_username" name="Instaberg_Plugin_Settings[proxy_username]" type="text" value="<?php echo esc_attr($proxy_info['proxy_username']); ?>" placeholder="username">
			<span class="Instaberg-form-note"><?php esc_html_e('the server username', 'instaberg'); ?></span>
		</div>

		<div>
			<input id="proxy_password" name="Instaberg_Plugin_Settings[proxy_password]" type="text" value="<?php echo esc_attr($proxy_info['proxy_password']); ?>" placeholder="password">
			<span class="Instaberg-form-note"><?php esc_html_e('the server password', 'instaberg'); ?></span>
		</div>

	</div>

</div>

							<?php
							$html[] = ob_get_clean();
							break;

						case 'box_apitesting':

							$apitest_method = $value['api_method'];
							$apitest_elem_id = $apitest_method . '_id';
							$apitest_elem_resp = $apitest_method . '_resp';

							$apitest_tab1 = $apitest_method . '_tab1';
							$apitest_tab2 = $apitest_method . '_tab2';

							$apitest_link = $value['api_link'];
							$apitest_postid = $value['api_postid'];
							$_apitest_sample_link = str_replace('{postid}', "<span>{$apitest_postid}</span>", $apitest_link);

							$apitesting_info = array(
								'id_value' => isset($settings["$apitest_elem_id"]) ? $settings["$apitest_elem_id"] : $apitest_postid,
							);

							// Striping slashes of non-array options
							foreach ($apitesting_info as $proxy_kk => $proxy_vv) {
								$apitesting_info["$proxy_kk"] = stripslashes( $proxy_vv );
							}

							ob_start();
							?>

<div class="Instaberg-form-row Instaberg_apitesting Instaberg_box_apitest" data-api_method="<?php echo esc_attr($apitest_method); ?>">

	<label for="Instaberg_box_apitest_content"><?php echo esc_html($value['title']); ?></label>

	<div class="Instaberg-form-item Instaberg_box_apitest_content">

		<div class="apitest_content1">

			<input id="<?php echo esc_attr($apitest_elem_id); ?>" name="Instaberg_apitesting[<?php echo esc_attr($apitest_elem_id); ?>]" type="text" value="<?php echo esc_attr($apitesting_info['id_value']); ?>" placeholder="<?php echo esc_attr($value['placeholder']); ?>">

			<input type="button" value="<?php esc_html_e('Run', 'instaberg'); ?>" class="Instaberg-button green Instaberg-action-run">
			<input type="button" value="<?php esc_html_e('Run Dev', 'instaberg'); ?>" class="Instaberg-button blue Instaberg-action-rundev">

			<span class="method_link">Ex.: <?php echo esc_html($_apitest_sample_link); ?></span>

		</div>

		<div class="apitest_content2">

			<ul class="Instaberg-tabs">
				<li class="on"><a href="#<?php echo esc_attr($apitest_tab1); ?>"><?php esc_html_e('Visual Response', 'instaberg'); ?></a></li>
				<li class=""><a href="#<?php echo esc_attr($apitest_tab2); ?>"><?php esc_html_e('Code Response', 'instaberg'); ?></a></li>
			</ul>

			<div class="Instaberg-tabs-content <?php echo esc_attr($apitest_tab1); ?>">
			  ...
			</div>

			<div class="Instaberg-tabs-content <?php echo esc_attr($apitest_tab2); ?>">
			  <textarea id="<?php echo esc_attr($apitest_elem_resp); ?>" name="Instaberg_apitesting[<?php echo esc_attr($apitest_elem_resp); ?>]"><?php esc_html_e('response here!', 'instaberg'); ?></textarea>
			</div>
		</div>

	</div>

</div>

							<?php
							$html[] = ob_get_clean();
							break;

						default:
							wp_die("Invalid filed types!");

					}

					// the element description
					if (isset($value['desc'])){
						$html[] = '<span class="Instaberg-form-note">' . ($value['desc']) . '</span>';
					}

					if (!in_array($value['type'], $noRowElements)) {
						// close: .Instaberg-form-row
						$html[] = '</div></div>';
					}
				}

				// Instaberg-message use for status message, default it's hidden
				$html[] = '<div class="Instaberg-message" id="Instaberg-status-box" style="display:none;"></div>';


				// buttons for saving settings
				$html[] = '<div class="Instaberg-button-row">
					<input type="submit" value="Save Settings" class="Instaberg-button yellow Instaberg-saveOptions" />
				</div>';

				// close: form
				$html[] = '</form>';

				// return the $html
				return implode("\n", $html);
			}
		}

		//make Tabs!
		private function tabs_header( $box ) {
			$html = array();

			// get tabs
			$tabs = isset($box['tabs']) ? $box['tabs'] : array();

			if (is_array($tabs) && count($tabs)>0) {
				$html[] = '<ul class="Instaberg-settings-tabs">';
				foreach ($tabs as $key_tab => $value_tab) {
					$html[] = '<li><a href="#' . ( $key_tab ) . '">' . ( $value_tab['label']  ) . '</a></li>';
				}
				$html[] = '</ul>';
			}

			// return the $html
			return implode("\n", $html);
		}

		private function tabs_elements($box, $elemKey) {
			// get tabs
			$__tabs = isset($box['tabs']) ? $box['tabs'] : array();

			$__ret = '';
			if (is_array($__tabs) && count($__tabs)>0) {
				foreach ($__tabs as $tabClass => $tabElements) {

					$tabElements = $tabElements['elements'];
					$tabElements = trim($tabElements);
					$tabElements = array_map('trim', explode(',', $tabElements));
					if (in_array($elemKey, $tabElements))
						$__ret .= ($tabClass.' '); //support element on multiple tabs!
				}
			}
			return ' '.trim($__ret).' ';
		}
	}
}
