<?php 

/*
 * Define class Instaberg_Template
 * Make sure you skip down to the end of this file, as there are a few
 * lines of code that are very important.
 */
!defined('ABSPATH') and exit;

if (class_exists('Instaberg_Template') != true) {
    
    class Instaberg_Template
    {
        /*
         * Some required plugin information
         */
        const VERSION = '1.0';
        
        public $post_type = '';
        public $cfg = array();
        public $template_directory = '';
        public $the_plugin = array();

        /*
         * Required __construct() function that initalizes the Instaberg_Template
         */
        public function __construct( $the_plugin=array() )
        {

            $this->the_plugin = $the_plugin;

            add_action( 'woocommerce_no_products_found', array($this, 'add_html_marker') );

            add_action( 'wp_enqueue_scripts', array( $this, 'add_styles' ), 10);

            add_action( 'wp_ajax_' . $this->the_plugin->alias . '_frontend_style', array( $this, 'frontend_style' ) );
            add_action( 'wp_ajax_nopriv_' . $this->the_plugin->alias . '_frontend_style', array( $this, 'frontend_style' ) );
            add_action( 'wp_ajax_' . $this->the_plugin->alias . '_save_stats', array( $this, 'save_stats' ) );

            add_action( 'wp_footer', array($this, 'not_found_box') );
        }

        public function add_styles()
        {
            wp_enqueue_style( Instaberg()->alias . '-Droid+Sans', 'https://fonts.googleapis.com/css?family=Droid+Sans:400,700', array(), '1.0' );
            wp_enqueue_style( Instaberg()->alias . '-rating', Instaberg()->assetUrl('star-rating-svg.css'), array(), '1.0' );

            wp_enqueue_script( Instaberg()->alias . '-amazon-rating', Instaberg()->assetUrl('jquery.star-rating-svg.js'), array( 'jquery' ) );

            wp_localize_script( Instaberg()->alias . '-amazon', Instaberg()->alias . '_custom', array(
                'ajaxurl' => admin_url('admin-ajax.php')
            ));

            wp_enqueue_style( Instaberg()->alias . '-frontend_style', admin_url( 'admin-ajax.php?action=' . Instaberg()->alias . '_frontend_style' ), array(), '1.0' );

            if( count($this->the_plugin->settings['setup']['template_typography']) > 0 ){
                $google_fonts = array();
                foreach ($this->the_plugin->settings['setup']['template_typography'] as $key => $value) {
                    $google_fonts[] = str_replace( " ", "+", $value['family']);
                }

                $google_fonts = array_unique( $google_fonts );
                wp_enqueue_style( Instaberg()->alias . '-googlefonts', '//fonts.googleapis.com/css?family=' . implode( "|", $google_fonts ), array(), '1.0' );
            }
        }

        public function get_client_ip() 
        {
            $ipaddress = '';

            if ($_SERVER['REMOTE_ADDR'])
                $ipaddress = $_SERVER['REMOTE_ADDR'];
            else if ($_SERVER['HTTP_CLIENT_IP'])
                $ipaddress = $_SERVER['HTTP_CLIENT_IP'];
            else if ($_SERVER['HTTP_X_FORWARDED'])
                $ipaddress = $_SERVER['HTTP_X_FORWARDED'];
            else if ($_SERVER['HTTP_FORWARDED_FOR'])
                $ipaddress = $_SERVER['HTTP_FORWARDED_FOR'];
            else if( $_SERVER['HTTP_FORWARDED'])
                $ipaddress = $_SERVER['HTTP_FORWARDED'];
            else if ($_SERVER['HTTP_X_FORWARDED_FOR'])
                $ipaddress = $_SERVER['HTTP_X_FORWARDED_FOR'];

            return $ipaddress;
        }

        public function ip2number( $ip ) 
        {
            $long = ip2long($ip);
            if ($long == -1 || $long === false) {
                return false;
            }
            return sprintf("%u", $long);
        }

        /**
         * Parameters: 
         *   $key - The key to be searched for.
         *   $list - The sorted array. 
         *   $compare_func - A user defined function for comparison. Same definition as the one in usort
         *   $low - First index of the array to be searched (local parameters).
         *   $high - Last index of the array to be searched (local parameters). 
         *
         * Return:
         *   index of the search key if found, otherwise return -(insert_index + 1). 
         *   insert_index is the index of greatest element that is smaller than $key or count($list) if $key
         *   is larger than all elements in the array.
         * 
         * License: Feel free to use the code if you need it.
         */
        public function binary_search($key, array $list, $compare_func) 
        {
            $low = 0; 
            $high = count($list) - 1;
     
            while ($low <= $high) {
                $mid = (int) (($high - $low) / 2) + $low; // could use php ceil function
                $cmp = call_user_func($compare_func, $list[$mid], $key);
     
                if ($cmp < 0) {
                    $low = $mid + 1;
                } else if ($cmp > 0) {
                    $high = $mid - 1;
                } else {
                    return $mid;
                }
            }
            return -($low - 1);
        }

        public function binary_search_cmp($a, $b) 
        {
            return ($a < $b) ? -1 : (($a > $b) ? 1 : 0);
        }


        public function frontend_style()
        {
            // Set the correct MIME type, because Apache won't set it for us
            header("Content-type: text/css");
            $settings = $this->the_plugin->settings['setup'];

            $css = array();

            // custom cases here ...
            if( isset($settings['products_per_column']) ){
                $css[] = "
                    body .Instaberg-amazon-response {
                        grid-template-columns: repeat(" . ( $settings['products_per_column'] ) . ", minmax(0, 1fr));
                    }
                ";
            }

            if( isset($settings['template_typography']['headline']) ){
                $css[] = "
                    body .Instaberg-box-title {
                        font-family: '" . ( $settings['template_typography']['headline']['family'] ) . "';
                        font-size: " . ( $settings['template_typography']['headline']['size'] ) . "px;
                        font-weight: " . ( $settings['template_typography']['headline']['weight'] ) . ";
                        color: " . ( $settings['template_typography']['headline']['color'] ) . ";
                    }
                ";
            }

            if( isset($settings['template_typography']['product_title']) ){
                $css[] = "
                    body .Instaberg-amazon-products .Instaberg-product .Instaberg-title {
                        font-family: '" . ( $settings['template_typography']['product_title']['family'] ) . "';
                        font-size: " . ( $settings['template_typography']['product_title']['size'] ) . "px;
                        font-weight: " . ( $settings['template_typography']['product_title']['weight'] ) . ";
                        color: " . ( $settings['template_typography']['product_title']['color'] ) . ";
                    }
                ";
            }

            if( isset($settings['template_typography']['product_sub_title']) ){
                $css[] = "
                    body .Instaberg-amazon-products .Instaberg-product .Instaberg-subtitle {
                        font-family: '" . ( $settings['template_typography']['product_sub_title']['family'] ) . "';
                        font-size: " . ( $settings['template_typography']['product_sub_title']['size'] ) . "px;
                        font-weight: " . ( $settings['template_typography']['product_sub_title']['weight'] ) . ";
                        color: " . ( $settings['template_typography']['product_sub_title']['color'] ) . ";
                    }
                ";
            }

            if( isset($settings['template_typography']['nb_of_reviews']) ){
                $css[] = "
                    body .Instaberg-amazon-products .Instaberg-product .Instaberg-rating-totals {
                        font-family: '" . ( $settings['template_typography']['nb_of_reviews']['family'] ) . "';
                        font-size: " . ( $settings['template_typography']['nb_of_reviews']['size'] ) . "px;
                        font-weight: " . ( $settings['template_typography']['nb_of_reviews']['weight'] ) . ";
                        color: " . ( $settings['template_typography']['nb_of_reviews']['color'] ) . ";
                    }
                ";
            }

            if( isset($settings['template_typography']['regular_price']) ){
                $css[] = "
                    body .Instaberg-amazon-products .Instaberg-product .Instaberg-price div {
                        font-family: '" . ( $settings['template_typography']['regular_price']['family'] ) . "';
                        font-size: " . ( $settings['template_typography']['regular_price']['size'] ) . "px;
                        font-weight: " . ( $settings['template_typography']['regular_price']['weight'] ) . ";
                        color: " . ( $settings['template_typography']['regular_price']['color'] ) . ";
                    }
                ";
            }


            if( isset($settings['template_typography']['sale_price']) ){
                $css[] = "
                    body .Instaberg-amazon-products .Instaberg-product .Instaberg-price .Instaberg-new-price {
                        font-family: '" . ( $settings['template_typography']['sale_price']['family'] ) . "';
                        font-size: " . ( $settings['template_typography']['sale_price']['size'] ) . "px;
                        font-weight: " . ( $settings['template_typography']['sale_price']['weight'] ) . ";
                        color: " . ( $settings['template_typography']['sale_price']['color'] ) . ";
                    }
                ";
            }

            if( isset($settings['template_typography']['buy_btn']) ){
                $css[] = "
                    body .Instaberg-amazon-products .Instaberg-product .Instaberg-button {
                        font-family: '" . ( $settings['template_typography']['buy_btn']['family'] ) . "';
                        font-size: " . ( $settings['template_typography']['buy_btn']['size'] ) . "px;
                        font-weight: " . ( $settings['template_typography']['buy_btn']['weight'] ) . ";
                        color: " . ( $settings['template_typography']['buy_btn']['color'] ) . ";
                        background-color: " . ( $settings['template_typography']['buy_btn']['background-color'] ) . ";
                    }
                ";
            }

            $buffer = implode( "\n", $css );

            // Remove comments
            $buffer = preg_replace('!/\*[^*]*\*+([^/][^*]*\*+)*/!', '', $buffer);
             
            // Remove space after colons
            $buffer = str_replace(': ', ':', $buffer);
             
            // Remove whitespace
            $buffer = str_replace(array("\r\n", "\r", "\n", "\t", '    ', '    '), '', $buffer);

            die( $buffer );
        }

        public function save_stats()
        {
            $params = isset($_REQUEST['params']) ? $_REQUEST['params'] : array();
            if( $params && count($params) > 0 ){
                $ip = $this->get_client_ip();

                Instaberg()->db->insert(
                    Instaberg()->db->prefix . 'Instaberg_stats',
                    array(
                        'action'        => isset($params['action']) ? $params['action'] : '',
                        'keyword'       => isset($params['keyword']) ? $params['keyword'] : '',
                        'amazon_site'   => isset($params['amzsite']) ? $params['amzsite'] : '',
                        'country'       => isset($params['country']) ? $params['country'] : '',
                        'url'           => isset($params['url']) ? $params['url'] : '',
                        'ip'            => $ip
                    ),
                    array(
                        '%s',
                        '%s',
                        '%s',
                        '%s',
                        '%s',
                        '%s'
                    )
                );
            }

            die("ok");
        }
    }
}