<?php
/**
 * Instagram Utils Class
 * http://www.aa-team.com
 * ======================
 *
 * @package		Instagram Utils Class
 * @author		Andrei Dinca, AA-Team
 * @version		1.0
 */
!defined('ABSPATH') and exit;
if (class_exists('Instaberg_Utils') != true) {
	class Instaberg_Utils
	{
		const VERSION = '1.0';

		// cached requests table
		private static $requests_cache_table = 'instaberg_requests_cache';

		// cached requests expiration mysql expression
		private static $requests_cache_expire = 'INTERVAL 24 HOUR';

		
		/**
		 * The constructor
		 */
		public function __construct()
		{
		}


		public function get_ip() 
		{
			//Just get the headers if we can or else use the SERVER global
			if ( function_exists( 'apache_request_headers' ) ) {
				$headers = apache_request_headers();
			} else {
				$headers = $_SERVER;
			}
			//Get the forwarded IP if it exists
			if ( array_key_exists( 'X-Forwarded-For', $headers ) && filter_var( $headers['X-Forwarded-For'], FILTER_VALIDATE_IP, FILTER_FLAG_IPV4 ) ) {
				$the_ip = $headers['X-Forwarded-For'];
			} elseif ( array_key_exists( 'HTTP_X_FORWARDED_FOR', $headers ) && filter_var( $headers['HTTP_X_FORWARDED_FOR'], FILTER_VALIDATE_IP, FILTER_FLAG_IPV4 )
			) {
				$the_ip = $headers['HTTP_X_FORWARDED_FOR'];
			} else {
				$the_ip = filter_var( $_SERVER['REMOTE_ADDR'], FILTER_VALIDATE_IP, FILTER_FLAG_IPV4 );
			}
			return $the_ip;
		}

		public function time_elapsed_string( $ptime )
		{
			$etime = time() - $ptime;
		
			if ($etime < 1)
			{
				return '0 seconds';
			}
		
			$a = array( 365 * 24 * 60 * 60  =>  'year',
						 30 * 24 * 60 * 60  =>  'month',
							  24 * 60 * 60  =>  'day',
								   60 * 60  =>  'hour',
										60  =>  'minute',
										 1  =>  'second'
						);
			$a_plural = array( 'year'   => 'years',
							   'month'  => 'months',
							   'day'    => 'days',
							   'hour'   => 'hours',
							   'minute' => 'minutes',
							   'second' => 'seconds'
						);
		
			foreach ($a as $secs => $str)
			{
				$d = $etime / $secs;
				if ($d >= 1)
				{
					$r = round($d);
					return $r . ' ' . ($r > 1 ? $a_plural[$str] : $str) . ' ago';
				}
			}
		}

		public function db_custom_insert( $table, $fields, $ignore=false, $wp_way=false ) {
			global $wpdb;
			
			if ( $wp_way && !$ignore ) {
				$wpdb->insert( 
					$table, 
					$fields['values'], 
					$fields['format']
				);
			} else {
				$formatVals = implode(', ', array_map( function($v) { return "'".$v."'"; }, $fields['format']));
				$theVals = array();
				foreach ( $fields['values'] as $k => $v ) $theVals[] = $k;

				$q = "INSERT " . ($ignore ? "IGNORE" : "") . " INTO $table (" . implode(', ', $theVals) . ") VALUES (" . $formatVals . ");";
				foreach ($fields['values'] as $kk => $vv)
					$fields['values']["$kk"] = esc_sql($vv);

				$q = vsprintf($q, $fields['values']);
				$r = $wpdb->query( $q );
			}
			return $wpdb->insert_id;
		}

		public function prepareForInList( $v ) {
			return "'".$v."'";
		}

		//format right (for db insertion) php range function!
		public function doRange( $arr ) {
			$newarr = array();
			if ( is_array($arr) && count($arr)>0 ) {
				foreach ($arr as $k => $v) {
					$newarr[ $v ] = $v;
				}
			}
			return $newarr;
		}


		// cURL / Send http requests with curl
		public static function curl( $url, $input_params=array(), $output_params=array(), $debug=false ) {

			$ret = array(
				'status' 		=> 'invalid',
				'msg' 			=> '',
				'http_code' 	=> 0,
				'data' 			=> ''
			);

			// build curl options
			$ipms = array_replace_recursive(array(
				'userpwd'                   => false,
				'htaccess'                  => false,
				'post'                      => false,
				'postfields'                => array(),
				'httpheader'                => false,
				'verbose'                   => false,
				'ssl_verifypeer'            => false,
				'ssl_verifyhost'            => false,
				'httpauth'                  => false,
				'failonerror'               => false,
				'returntransfer'            => true,
				'binarytransfer'            => false,
				'header'                    => false,
				'cainfo'                    => false,
				'useragent'                 => false,
				'followlocation' 			=> false,
				'connecttimeout' 			=> false,
				'timeout' 					=> false,
				'proxy' 					=> false,
				'proxy_userpwd' 			=> null,
			), $input_params);
			extract($ipms);

			$opms = array_replace_recursive(array(
				'resp_is_json'              => false,
				'resp_add_http_code'        => false,
				'parse_headers'             => false,
				'find_cookies' 				=> false,
			), $output_params);
			extract($opms);

			// begin curl
			$url = trim($url);
			if ( empty($url) ) {
				return array_replace_recursive( $ret, array(
					'msg' 	=> 'url is empty!',
				));
			}

			$curl = curl_init();

			$curl_options = array();
			$curl_options[CURLOPT_URL] = $url;

			if ( ! empty($userpwd) ) {
				$curl_options[CURLOPT_USERPWD] = $userpwd;
			}

			if ( ! empty($htaccess) ) {
				$url = preg_replace( "/http(|s):\/\//i", "http://" . $htaccess . "@", $url );
			}

			if ( $post ) {
				$curl_options[CURLOPT_POST] = true;
				$curl_options[CURLOPT_POSTFIELDS] = $postfields;
			}
			else if ( ! $post && ! empty($postfields) ) {
				$url = $url . "?" . http_build_query($postfields);
			}

			if ( !empty($httpheader) ) {
				$curl_options[CURLOPT_HTTPHEADER] = $httpheader;
			}
						
			if ( $verbose !== false ) 			$curl_options[CURLOPT_VERBOSE] = $verbose;
			if ( $ssl_verifypeer !== false ) 	$curl_options[CURLOPT_SSL_VERIFYPEER] = $ssl_verifypeer;
			if ( $ssl_verifyhost !== false ) 	$curl_options[CURLOPT_SSL_VERIFYHOST] = $ssl_verifyhost;
			if ( $httpauth !== false ) 			$curl_options[CURLOPT_HTTPAUTH] = $httpauth;
			if ( $failonerror !== false ) 		$curl_options[CURLOPT_FAILONERROR] = $failonerror;
			if ( $returntransfer !== false ) 	$curl_options[CURLOPT_RETURNTRANSFER] = $returntransfer;
			if ( $binarytransfer !== false ) 	$curl_options[CURLOPT_BINARYTRANSFER] = $binarytransfer;
			if ( $header !== false ) 			$curl_options[CURLOPT_HEADER] = $header;
			if ( $cainfo !== false ) 			$curl_options[CURLOPT_CAINFO] = $cainfo;
			if ( $useragent !== false ) 		$curl_options[CURLOPT_USERAGENT] = $useragent;
			if ( $followlocation !== false ) 	$curl_options[CURLOPT_FOLLOWLOCATION] = $followlocation;
			if ( $connecttimeout !== false ) 	$curl_options[CURLOPT_CONNECTTIMEOUT] = $connecttimeout;
			if ( $timeout !== false ) 			$curl_options[CURLOPT_TIMEOUT] = $timeout;

			//PROXYUSERPWD
			if ( $proxy !== false ) {
				$curl_options[CURLOPT_PROXY] = $proxy;
				$curl_options[CURLOPT_PROXYUSERPWD] = $proxy_userpwd;
			}

			curl_setopt_array($curl, $curl_options);

			$data = curl_exec($curl);
			$http_code = (int) curl_getinfo($curl, CURLINFO_HTTP_CODE);

			$ret = array_merge($ret, array('http_code' => $http_code, 'http_message' => ''));
			if ($debug) {
				$ret = array_merge($ret, array('debug_details' => curl_getinfo($curl)));
			}
			// error occurred
			if ( $data === false || curl_errno($curl) ) {
				$__ = curl_errno($curl) . ' : ' . curl_error($curl);
				$ret = array_merge($ret, array(
					'msg' 	=> $__,
					'data' 	=> $__,
				));
			}
			// success
			else {

				if ( $parse_headers ) {
					$header_size = curl_getinfo($curl, CURLINFO_HEADER_SIZE);
					$headers_parsed = self::parse_headers( substr($data, 0, $header_size), $find_cookies ); // response begin with the headers
					$data = substr($data, $header_size);
					$ret = array_merge($ret, array(
						'headers' => $headers_parsed['headers'],
						'cookies' => $headers_parsed['cookies'],
					));
				}

				// Add the status code to the json data, useful for error-checking
				if ( $resp_add_http_code && $resp_is_json ) {
					$data = preg_replace('/^{/', '{"http_code":'.$http_code.',', $data);
				}

				$ret = array_merge($ret, array(
					'status' 	=> 'valid',
					'msg' 		=> 'ok',
					'data' 		=> $data
				));
			}

			curl_close($curl);
			return $ret;
		}

		// fsocket / Send http requests with fsocket
		public static function fsocket( $url, $input_params=array(), $output_params=array(), $debug=false ) {

			$ret = array(
				'status' 		=> 'invalid',
				'msg' 			=> '',
				'http_code' 	=> 0,
				'data' 			=> ''
			);

			// build curl options
			$ipms = array_replace_recursive(array(
				'http_method' 		=> 'GET',
				'urlparts'          => array(),
				'headers'           => array(),
				'postfields'        => '',
			), $input_params);
			extract($ipms);

			$opms = array_replace_recursive(array(
				'resp_is_json'              => false,
				'resp_add_http_code'        => false,
				'parse_headers'             => false,
				'find_cookies' 				=> false,
			), $output_params);
			extract($opms);

			$sep = "\r\n";

			$headers_str = implode($sep, $headers);

			$datawrite = sprintf(
				"%s %s HTTP/1.1{$sep}%s{$sep}{$sep}%s",
				$http_method,
				$urlparts['path'] . ( ! empty($urlparts['query']) ? '?' . $urlparts['query'] : '' ),
				$headers_str,
				$postfields
			);

			$scheme = $urlparts['scheme'];
			$port = 80;
			if ( 'https' === $scheme) {
				$scheme = 'ssl';
				$port = ! empty($urlparts['port']) ? $urlparts['port'] : 443;
			}

			$scheme = ! empty($scheme) ? $scheme . '://' : '';

			$fp = @fsockopen( $scheme . $urlparts['host'], $port, $errno, $errstr, 20 );

			//:: error occurred
			if ( ! $fp ) {
				$__ = 'fsocket error occured; err number : ' . $errno . '; err details : ' . $errstr;
				$ret = array_merge($ret, array(
					'msg' 	=> $__, 
					'data' 	=> $__,
				));
				return $ret;
			}

			//:: success
			// Add the status code to the json data, useful for error-checking
			fwrite($fp, $datawrite);

			// read data list
			$response = '';
			while ( $line = fgets($fp, 1024) ) {
				$response .= $line;
			}

			fclose($fp);

			// find body & headers from response
			@list( $headers_resp, $body_encoded_resp, $altbody_encoded_resp ) = explode("{$sep}{$sep}", $response);
			if ( $altbody_encoded_resp ) {
				$headers_resp = $body_encoded_resp;
				$body_encoded_resp = $altbody_encoded_resp;
			}
			
			$headers_resp_arr = explode("$sep", $headers_resp);
			
			$http_code_str = array_shift($headers_resp_arr);
			$http_code_arr = explode(' ', substr($http_code_str, 0, 50));
			$http_code = isset($http_code_arr[1]) ? (int) $http_code_arr[1] : 0;
			$http_message = isset($http_code_arr[2]) ? trim( $http_code_arr[2] ) : '';
			
			$ret = array_merge($ret, array('http_code' => $http_code, 'http_message' => $http_message));

			$data = $body_encoded_resp;

			if ( $parse_headers ) {
				$headers_parsed = self::parse_headers( $headers_resp_arr, $find_cookies ); // response begin with the headers
				$ret = array_merge($ret, array(
					'headers' => $headers_parsed['headers'],
					'cookies' => $headers_parsed['cookies'],
				));
			}

			// Add the status code to the json data, useful for error-checking
			if ( $resp_add_http_code && $resp_is_json ) {
				$data = preg_replace('/^{/', '{"http_code":'.$http_code.',', $data);
			}

			$ret = array_merge($ret, array(
				'status' 	=> 'valid',
				'msg' 		=> 'ok',
				'data' 		=> $data
			));
			return $ret;
		}

		private static function parse_headers( $headers, $find_cookies=false ) {

			if ( ! is_array($headers) ) {
				$headers = explode("\r\n", $headers);
			}

			$ret = array(
				'headers' => array(),
				'cookies' => array()
			);

			foreach ( $headers as $header ) {

				$header = explode(":", $header, 2);
				if ( count($header) != 2 ) {
					continue 1;
				}

				$header_key = $header[0];
				$header_val = trim($header[1]);

				if ( 'set-cookie' === strtolower($header_key) ) {
					$cookie_params = explode(';', $header_val);

					if ( empty($cookie_params[0]) ) {
						continue 1;
					}

					$cookie = explode('=', $cookie_params[0]);
					if ( count($cookie) != 2 ) {
						continue 1;
					}

					$cookie_name = $cookie[0];
					$cookie_val = trim($cookie[1]);

					$ret['cookies'][$cookie_name] = $cookie_val;

				}
				else {
					$ret['headers'][$header_key] = $header_val;
				}
			}
			return $ret;
		}


		// database basic cache system
		public function requests_create_table() {
			global $wpdb;

			// install db /check if table exist, if not create table
			$table_name = $wpdb->prefix . self::$requests_cache_table;

			if ( $wpdb->get_var( "show tables like '$table_name'" ) != $table_name ) {

				$sql = "
					CREATE TABLE IF NOT EXISTS " . $table_name . " (
						`ID` BIGINT(20) UNSIGNED NOT NULL AUTO_INCREMENT,
						`cache_name` VARCHAR(100) NOT NULL,
						`cache_type` VARCHAR(40) NOT NULL,
						`response` LONGTEXT NOT NULL,
						`response_date` TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
						PRIMARY KEY (`ID`),
						UNIQUE INDEX `cache_name_cache_type` (`cache_name`, `cache_type`),
						INDEX `cache_type` (`cache_type`),
						INDEX `response_date` (`response_date`)
					)
					COLLATE='utf8_general_ci'
					ENGINE=InnoDB;
					";

				require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
				dbDelta($sql);
			}
		}

		public function request_get_cache( $cache_name, $cache_type, $pms=array() ) {
			global $wpdb;

			$pms = array_replace_recursive(array(
				'check_expire' 	=> true,
			), $pms);
			extract( $pms );

			$table = $wpdb->prefix . self::$requests_cache_table;
			$how_often = self::$requests_cache_expire; //'INTERVAL 1 HOUR';

			$sql_check_expire = " and ( response_date > DATE_SUB( NOW(), $how_often ) )";
			if ( ! $check_expire ) {
				$sql_check_expire = '';
			}

			if ( is_array($cache_name) ) {
				$cache_name_ = implode(',', array_map(array($this, 'prepareForInList'), $cache_name));
				$sql = "select cache_name, response from $table where 1=1 and cache_name in ( $cache_name_ ) and cache_type = %s $sql_check_expire;";
				$sql = $wpdb->prepare( $sql, $cache_type );
				$res = $wpdb->get_results( $sql, OBJECT_K );
				if ( empty($res) ) {
					return array();
				}

				$ret = array();
				foreach ($res as $key => $val) {
					$__ = $val->response;
					//$__ = maybe_unserialize( $__ );
					$__ = json_decode( $__, true );
					$ret["$key"] = $__;
				}
				return $ret;
			}
			else {
				$sql = "select response from $table where 1=1 and cache_name = %s and cache_type = %s $sql_check_expire limit 1;";
				$sql = $wpdb->prepare( $sql, $cache_name, $cache_type );
				$res = $wpdb->get_var( $sql );
				if ( empty($res) ) {
					return $res;
				}

				$ret = $res;
				//$ret = maybe_unserialize( $ret );
				$ret = json_decode( $ret, true );
				return $ret;
			}
			return false;
		}

		public function request_save_cache( $cache_name, $cache_type, $pms=array() ) {
			global $wpdb;

			$pms = array_replace_recursive(array(
				'response' 	=> array(),
			), $pms);
			extract( $pms );

			$table = $wpdb->prefix . self::$requests_cache_table;

			// delete old cache
			$sql = "delete from $table where 1=1 and cache_name = %s and cache_type = %s;";
			$sql = $wpdb->prepare( $sql, $cache_name, $cache_type );
			$res = $wpdb->query( $sql );

			// insert (ignore) new response as cache
			$response = json_encode( $response );
			$this->db_custom_insert(
				$table,
				array(
					'values' => array(
						'cache_name' 	=> $cache_name,
						'cache_type' 	=> $cache_type,
						'response' 		=> $response,
					),
					'format' => array(
						'%s', '%s', '%s'
					)
				),
				true
			);
			return true;
		}
	}
}