<?php
/*
Plugin Name:	Instaberg - Instagram Feed Gallery - Gutenberg Block
Plugin URI: 	http://codecanyon.net/user/AA-Team/portfolio
Description: 	Instaberg is the newest Instagram feed plugin for WordPress that allows you to display posts/videos from your own Instagram account /  public posts into any page/post by using WordPress blocks!
Version: 		1.0
Author: 		AA-Team
Author URI:		http://codecanyon.net/user/AA-Team/portfolio
*/

// don't load directly
if ( ! defined( 'ABSPATH' ) ) {
	die( '-1' );
}
/**
 * Current Instaberg version
 */
if ( ! defined( 'Instaberg_VERSION' ) ) {
	/**
	 *
	 */
	define( 'Instaberg_VERSION', '1.0.0' );
}
/**
 * Instaberg starts here. Manager sets mode, adds required wp hooks and loads required object of structure
 *
 * Manager controls and access to all modules and classes of Instaberg.
 *
 * @package ' . ( $this->plugin_name ) . '
 * @since   1.0
 */
class Instaberg {
	/**
	 * Set status/mode for Instaberg.
	 *
	 * It depends on what functionality is required from Instaberg to work with current page/part of WP.
	 *
	 * Possible values:
	 *  none - current status is unknown, default mode;
	 *  page - simple wp page;
	 *  admin_page - wp dashboard;
	 *  admin_frontend_editor - Instaberg front end editor version;
	 *  admin_settings_page - settings page
	 *  page_editable - inline version for iframe in front end editor;
	 *
	 * @since 1.0
	 * @var string
	 */
	private $mode = 'none';

	/**
	 * Enables Instaberg to act as the theme plugin.
	 *
	 * @since 1.0
	 * @var bool
	 */

	private $is_as_theme = false;
	/**
	 * Instaberg is network plugin or not.
	 * @since 1.0
	 * @var bool
	 */
	private $is_network_plugin = null;

	/**
	 * List of paths.
	 *
	 * @since 1.0
	 * @var array
	 */
	private $paths = array();

	/**
	 * Set updater mode
	 * @since 1.0
	 * @var bool
	 */
	private $disable_updater = false;

	/**
	 * Modules and objects instances list
	 * @since 1.0
	 * @var array
	 */
	private $factory = array();

	/**
	 * File name for components manifest file.
	 *
	 * @since 4.4
	 * @var string
	 */
	private $components_manifest = 'components.json';

	/**
	 * @var string
	 */
	public $plugin_name = 'Instaberg';
	public $localizationName = 'Instaberg';
	public $alias = 'Instaberg';


	/**
	 * The tmdb object
	 */
	public $config = null;

	public $about = null;

	public $dashboard = null;

	/**
	 * The template_config object
	 */
	public $template_config = null;

	/**
	 * The wp_filesystem object
	 */
	public $wp_filesystem = null;

	/**
	 * The wpbd object
	 */
	public $db = null;

	/**
	 * The template object
	 */
	public $template = null;

	public $settings = array();

	public $amz_setup = array();

	public $amz_cates = array();


	/**
	 * Constructor loads API functions, defines paths and adds required wp actions
	 *
	 * @since  1.0
	 */
	public function __construct()
	{
		$app_dir = dirname( __FILE__ ); //no end slash

		$app_url = plugin_dir_url( __FILE__ ); //contains end slash
		$app_url = rtrim($app_url, '/'); //no end slash

		$upload_dir = wp_upload_dir();

		/**
		 * Define path settings for Instaberg.
		 */
		$this->setPaths( array(
			'WP_ROOT' 				=> preg_replace( '/$\//', '', ABSPATH ),
			'APP_ROOT' 				=> $app_dir,
			'APP_URL'  				=> $app_url,
			'APP_DIR_NAME'			=> basename( $app_dir ),

			'UPLOAD_BASE_DIR'  		=> $upload_dir['basedir'], //no end slash
			'UPLOAD_BASE_URL'  		=> $upload_dir['baseurl'], //no end slash

			'ASSETS_DIR_NAME' 		=> 'assets',
			'INCLUDE_DIR_NAME' 		=> 'include',

			'INCLUDE_DIR' 			=> $app_dir . '/include',
			'TEMPLATES_DIR' 		=> $app_dir . '/templates',
			'HELPERS_DIR' 			=> $app_dir . '/include/helpers',
			'DASHBOARD_DIR' 		=> $app_dir . '/include/dashboard',
			'ABOUT_DIR' 			=> $app_dir . '/include/about',
			'CONFIG_DIR' 			=> $app_dir . '/include/config',
			'APITESTING_DIR' 		=> $app_dir . '/include/apitesting',

			'ASSETS_DIR' 			=> $app_dir . '/assets',
			'ASSETS_URL' 			=> $app_url . '/assets',

			'BUILD_DIR' 			=> $app_dir . '/build',
			'BUILD_URL' 			=> $app_url . '/build',
		) );

		// Load API
		require_once $this->path( 'HELPERS_DIR', 'utils.php' );
		require_once $this->path( 'HELPERS_DIR', 'helpers.php' );
		require_once $this->path( 'HELPERS_DIR', 'settings.class.php' );
		require_once $this->path( 'HELPERS_DIR', 'instagram.api.parse.php' );
		InstabergApiParse();

		// Add hooks
		add_action( 'plugins_loaded', array( $this, 'pluginsLoaded' ), 9 );
		add_action( 'init', array( $this, 'init' ) );

		// load WP_Filesystem
		include_once ABSPATH . 'wp-admin/includes/file.php';
		WP_Filesystem();
		global $wp_filesystem;
		$this->wp_filesystem = $wp_filesystem;

		register_activation_hook( __FILE__, array( $this, 'install' ) );

		// create AJAX request
		add_action('wp_ajax_Instaberg_register', array(
			$this,
			'ajax_register'
		));

		add_action( 'init', array($this, 'init_session') );

		add_action( 'wp_ajax_Instaberg_action', array( $this, 'ajax_request' ) );
		add_action( 'wp_ajax_nopriv_Instaberg_action', array( $this, 'ajax_request' ) );
	}

	/**
	 * Callback function WP plugin_loaded action hook. Loads locale
	 *
	 * @since  1.0
	 * @access public
	 */
	public function pluginsLoaded()
	{
		// Setup locale
		do_action( 'Instaberg_plugins_loaded' );
		load_plugin_textdomain( 'instaberg', false, $this->path( 'APP_DIR_NAME', 'languages' ) ); //locale | languages
	}

	public function init_session()
	{
		 if( !session_id() ) {
			session_start();
		}
	}

	/**
	 * Callback function for WP init action hook. Sets Instaberg mode and loads required objects.
	 *
	 * @since  1.0
	 * @access public
	 *
	 * @return void
	 */
	public function init()
	{
		do_action( 'Instaberg_before_init' );

		wp_set_script_translations( 'instaberg/frontend.build', 'instaberg' );
		wp_set_script_translations( 'instaberg/backend.build', 'instaberg' );

		$this->update_developer();

		global $wpdb;
		$this->db = $wpdb;

		// Set current mode
		$this->setMode();

		/**
		 * if is admin and not frontend.
		 */
		if( $this->mode === 'admin' ) {

			// Load the admin menu hook
			$this->adminInterface();

			// If the user can manage options, let the fun begin!
			if ( current_user_can( 'manage_options' ) ) {
				// Adds actions to hook in the required css and javascript
				add_action( "admin_print_styles", array( &$this, 'admin_load_styles') );
				add_action( "admin_print_scripts", array( &$this, 'admin_load_scripts') );
			}

			// redirect on install to settings page
			$redirect_url = get_option( 'Instaberg_redirect_to' );
			if( $redirect_url ){
				delete_option( 'Instaberg_redirect_to' );
				wp_redirect( $redirect_url, 301 ); exit;
			}

			// load config interface
			require_once $this->path( 'CONFIG_DIR', 'config.class.php' );
			$this->config = new Instaberg_config( $this );

			// load about interface
			require_once $this->path( 'ABOUT_DIR', 'about.class.php' );
			$this->about = new Instaberg_about( $this );

			// load about interface
			require_once $this->path( 'DASHBOARD_DIR', 'dashboard.class.php' );
			$this->dashboard = new Instaberg_dashboard( $this );

			// load api testing interface
			require_once $this->path( 'APITESTING_DIR', 'apitesting.class.php' );
			$this->apitesting = new Instaberg_apitesting( $this );
		}
		else{
			$this->frontend_assets();
		}

		do_action( 'Instaberg_after_init' );
	}


	private function load_custom_post_types()
	{
		foreach( glob( $this->path( 'APP_ROOT', "/post-types/*/init.class.php" ) ) as $pt_init ){
			$this->cfg['CURRENT_PT_PATH'] = $pt_init;
			$this->cfg['CURRENT_PT_URL'] = plugin_dir_url( $this->cfg['CURRENT_PT_PATH'] );

			$GLOBALS['Instaberg'] = $this;
			require_once $pt_init;
		}
	}

	public function add_cron_recurrence_interval( $schedules )
	{
		$schedules['Instaberg_every_three_minutes'] = array(
			'interval'  => 180,
			'display'   => esc_html__( 'Every 3 Minutes', 'instaberg' )
		);

		return $schedules;
	}

	public function install()
	{
		global $wpdb;

		InstabergUtils()->requests_create_table();

		$options = get_option( 'Instaberg_Plugin_Settings' );

		if( !$options && !is_array($options) ){
			$file = $this->path( 'ASSETS_DIR', 'default.json' );
			$default = json_decode( $this->wp_filesystem->get_contents( $file ), true);
			add_option( 'Instaberg_Plugin_Settings', $default );

		}
	}

	public function update_developer()
	{
		//return true;
		if ( in_array($_SERVER['REMOTE_ADDR'], array()) ) {
			$this->dev = 'andrei';
		} else {
			$this->dev = 'gimi';
		}
	}

	/**
	 * Load required logic for operating in Wp Admin dashboard.
	 *
	 * @since  1.0
	 * @access protected
	 *
	 * @return void
	 */
	public function adminInterface()
	{
		// Settings page. Adds menu page in admin panel.
		$this->addMenuPageHooks();

	}

	public function addMenuPageHooks()
	{
		$this->editor_assets();

		if ( current_user_can( 'manage_options' ) ) {
			add_action( 'admin_menu', array( &$this, 'addMenuPage' ) );
		}
	}

	public function addMenuPage()
	{
		$page = add_menu_page( esc_html__( "Instaberg Feed", 'instaberg' ),
			esc_html__( "Instaberg Feed", 'instaberg' ),
			'manage_options',
			'Instaberg',
			array( &$this, 'render' ),
			Instaberg_asset_url( 'images/icon.png' )
		);

		add_action( "admin_print_styles", array( &$this, 'adminLoad' ) );
		add_action( "admin_head", array( &$this, 'adminLoadScript' ) );
	}

	/**
	 * Set up the enqueue for the CSS & JavaScript files.
	 *
	 */
	public function adminLoad()
	{
		wp_enqueue_style( 'Instaberg-core', Instaberg_asset_url( 'style.css' ), array(), Instaberg_VERSION );
		wp_enqueue_style( 'Instaberg-settings', Instaberg_asset_url( 'settings.css' ), array(), Instaberg_VERSION );

		wp_enqueue_style( 'Instaberg-font-awesome', Instaberg_asset_url( 'font-awesome.min.css' ), array(), '4.3.0' );

		wp_enqueue_script( 'Instaberg-script', Instaberg_asset_url( 'admin.js' ), array(), '1.0.0' );
	}

	public function adminLoadScript()
	{
	}

	public function editor_assets()
	{
		wp_enqueue_style(
			'instaberg/frontend.view',
			$this->path( 'BUILD_URL', 'frontend/frontend.view.css' ),
			array( 'wp-edit-blocks' )
		);

		wp_enqueue_style(
			'instaberg/backend.editor',
			$this->path( 'BUILD_URL', 'backend/backend.editor.css' ),
			array( 'wp-edit-blocks' )
		);

		wp_enqueue_script(
			'instaberg/backend.build',
			$this->path( 'BUILD_URL', 'backend/backend.build.js' ),
			array( 'wp-components', 'wp-blocks', 'wp-element', 'wp-editor', 'wp-i18n', 'wp-edit-post', 'lodash' )
		);

		wp_localize_script( 'instaberg/backend.build', 'INSTABERG_VARS', array(
			'ajax_url' 	=> apply_filters( 'Instaberg_ajax_request', esc_url( admin_url('admin-ajax.php?action=Instaberg_InstagramApiParse') ) ),
			'buy_url' 	=> '',
			'assets_url' => $this->path( 'ASSETS_URL', '/' ),
			'fonts' => array(),
		) );

		wp_enqueue_script(
			'instaberg/typo.build',
			$this->path( 'BUILD_URL', 'typo/typo.build.js' ),
			array( 'wp-components', 'wp-blocks', 'wp-element', 'wp-editor', 'wp-i18n', 'wp-edit-post', 'lodash' )
		);

		wp_enqueue_script( 'jquery-ui-core' );
		wp_enqueue_script( 'jquery-ui-sortable' );
	}

	public function frontend_assets()
	{
		wp_enqueue_style(
			'instaberg/frontend.view',
			$this->path( 'BUILD_URL', 'frontend/frontend.view.css' ),
			array( 'wp-edit-blocks' )
		);

		wp_enqueue_script(
			'instaberg/frontend.build',
			$this->path( 'BUILD_URL', 'frontend/frontend.build.js' ),
			array( 'wp-components', 'jquery' )
		);

		wp_localize_script( 'instaberg/frontend.build', 'INSTABERG_VARS', array(
			'assets_url' => $this->path( 'ASSETS_URL', '/' ),
			'fonts' => array(),
		) );
	}


	private function output_menu_section( $menu_arr=array() )
	{
		$html = array();
		if( count($menu_arr) > 0 ){

			$app_root = esc_url( admin_url( 'admin.php?page=Instaberg' ) );
			foreach ($menu_arr as $menu_key => $value) {

				$campaign_id = 'aa';
				$html[] = '<div class="Instaberg_iw-menu-section ' . ( $campaign_id == $menu_key ? 'menu_is_open' : '' ) . '">';

				if( count($value['links']) > 0 ){
					foreach ($value['links'] as $link_key => $link_value) {
						$link_value['url'] = str_replace( '%app_root%', $app_root, $link_value['url'] );

						$selected = "";
						if( Instaberg_action() == $link_key && $campaign_id == $menu_key ){
							$selected = 'on';
						}
						$html[] = '<a href="' . ( $link_value['url'] ) . '" class="' . ( $selected ) . '"><i class="fa ' . ( $link_value['icon'] ) . '"></i> ' . ( ucfirst( $link_value['label'] ) ) . '</a>';

						if( isset($link_value['submenu']) && count($link_value['submenu']) > 0 ){
							foreach ($link_value['submenu'] as $submenu_key => $submenu_value) {
								$submenu_value['url'] = str_replace( '%app_root%', $app_root, $submenu_value['url'] );

								$selected = "";
								if( Instaberg_action() == $submenu_key && $campaign_id == $menu_key ){
									$selected = 'on';
								}
								$html[] = '<a href="' . ( $submenu_value['url'] ) . '" class="Instaberg-submenu ' . ( $selected ) . '"><i class="fa fa-long-arrow-right"></i> ' . ( ucfirst( $submenu_value['label'] ) ) . '</a>';
							}
						}
					}

				}
				$html[] = '';
				$html[] = '</div>';
			}
		}

		return implode( "\n", $html );
	}

	/**
	 * Create Render points.
	 *
	 * Loaded interface depends on which page is requested by client from server and request parameters like Instaberg_action.
	 *
	 * @since  1.0
	 * @access protected
	 *
	 * @return void
	 */
	public function render()
	{
		$html = array();

		$html[] = '<div class="Instaberg_iw">';
		$html[] = '<div class="Instaberg_iw-loader" style="display: block;"><ul class="Instaberg_iw-preloader"><li></li><li></li><li></li><li></li><li></li></ul><span></span></div>';

		$html[] = '<aside>';
		$html[] = 	'<a id="Instaberg_iw-logo" href="' . ( esc_url( admin_url( 'admin.php?page=Instaberg' ) ) ) . '"><img src="' . ( Instaberg_asset_url('images/aa-logo.png') ) . '" /></a>';

		$extra_menu = array();

		$extra_menu[] = array(
			'label' 	=> '',
			'links'		=> array(
				'dashboard' => array(
					'url' 	=> '%app_root%&Instaberg_action=dashboard',
					'label'	=> esc_html__('Dashboard', 'instaberg'),
					'icon'	=> 'fa-tachometer'
				),

				'config' => array(
					'url' 	=> '%app_root%&Instaberg_action=config',
					'label'	=> esc_html__('Configuration', 'instaberg'),
					'icon'	=> 'fa-cog'
				),

				'apitesting' => array(
					'url' 	=> '%app_root%&Instaberg_action=apitesting',
					'label'	=> esc_html__('Instagram API Test', 'instaberg'),
					'icon'	=> 'fa-cog'
				),

				'about' => array(
					'url' 	=> '%app_root%&Instaberg_action=about',
					'label'	=> esc_html__('About', 'instaberg'),
					'icon'	=> 'fa-question'
				)
			)
		);


		$html[] = 	$this->output_menu_section($extra_menu);


		$html[] = '<span id="Instaberg-version">' . esc_html('Version', 'instaberg') .': ' . ( Instaberg_VERSION ) . ' by <a href="http://www.aa-team.com">AA-Team</a></span>';

		$html[] = '</aside>';

		$html[] = '<section>';

		if( Instaberg_action() == 'config' ){

			$html[] = 	'
				<div class="Instaberg_iw-header">
					<h3>' . esc_html__('Configuration', 'instaberg') . '</h3>
				</div>';

			$html[] = $this->config->print_interface();
		}

		else if( Instaberg_action() == 'about' ){
			$html[] = 	'
				<div class="Instaberg_iw-header">
					<h3>' . esc_html__('About', 'instaberg') . '</h3>
				</div>';

			$html[] = $this->about->print_interface();
		}

		else if( Instaberg_action() == 'dashboard' ){
			$html[] = 	'
				<div class="Instaberg_iw-header">
					<h3>' . esc_html__('Dashboard', 'instaberg') . '</h3>
				</div>';
				
			$html[] = $this->dashboard->print_interface();
		}

		if( Instaberg_action() == 'apitesting' ){

			$html[] = 	'
				<div class="Instaberg_iw-header">
					<h3>' . esc_html__('Instagram API Test', 'instaberg') . '</h3>
				</div>';

			$html[] = $this->apitesting->print_interface();
		}

		$html[] = '</section>';

		$html[] = '</div>';


		echo implode( "\n", $html );
	}

	/**
	 * Print Instaberg interface
	 *
	*/
	public function print_interface()
	{
		$html = array();

	}

	/**
	 * Set Instaberg mode.
	 *
	 * Mode depends on which page is requested by client from server and request parameters like Instaberg_action.
	 *
	 * @since  1.0
	 * @access protected
	 *
	 * @return void
	 */
	protected function setMode()
	{
		if ( is_admin() ) {
			$this->mode = 'admin';
		} else {
			$this->mode = 'frontend';
		}
	}

	/**
	 * Sets version of the Instaberg in DB as option `Instaberg_VERSION`
	 *
	 * @since 1.0
	 * @access protected
	 *
	 * @return void
	 */
	protected function setVersion()
	{
		$version = get_option( 'Instaberg_VERSION' );
		if ( ! is_string( $version ) || version_compare( $version, Instaberg_VERSION ) !== 0 ) {
			add_action( 'Instaberg_after_init', array( Instaberg_settings(), 'rebuild' ) );
			update_option( 'Instaberg_VERSION', Instaberg_VERSION );
		}
	}

	/**
	 * Get current mode for Instaberg.
	 *
	 * @since  1.0
	 * @access public
	 *
	 * @return string
	 */
	public function mode() {
		return $this->mode;
	}

	/**
	 * Setter for paths
	 *
	 * @since  1.0
	 * @access protected
	 *
	 * @param $paths
	 */
	protected function setPaths( $paths ) {
		$this->paths = $paths;
	}

	/**
	 * Gets absolute path for file/directory in filesystem.
	 *
	 * @since  1.0
	 * @access public
	 *
	 * @param $name - name of path dir
	 * @param string $file - file name or directory inside path
	 *
	 * @return string
	 */
	public function path( $name, $file = '' ) {
		$path = $this->paths[ $name ] . ( strlen( $file ) > 0 ? '/' . preg_replace( '/^\//', '', $file ) : '' );

		return apply_filters( 'Instaberg_path_filter', $path );
	}

	/**
	 * Returns list of default post types where user can use Instaberg editors.
	 *
	 * @since  1.0
	 * @access public
	 *
	 * @return array
	 */
	public function editorDefaultPostTypes() {
		return $this->editor_default_post_types;
	}

	/**
	 * Directory name where template files will be stored.
	 *
	 * @since  1.0
	 * @access public
	 *
	 * @return string
	 */
	public function uploadDir()
	{
		return 'aa-backup-manager';
	}

	/**
	 * Getter for plugin name variable.
	 * @since 1.0
	 *
	 * @return string
	 */
	public function pluginName()
	{
		return $this->plugin_name;
	}

	/**
	 * Get absolute url for Instaberg asset file.
	 *
	 * Assets are css, javascript, less files and images.
	 *
	 * @since 4.2
	 *
	 * @param $file
	 *
	 * @return string
	 */
	public function includeUrl( $file )
	{
		return preg_replace( '/\s/', '%20', plugins_url( $this->path( 'INCLUDE_DIR_NAME', $file ), __FILE__  ) );
	}

	/**
	 * Get absolute url for Instaberg asset file.
	 *
	 * Assets are css, javascript, less files and images.
	 *
	 * @since 4.2
	 *
	 * @param $file
	 *
	 * @return string
	 */
	public function assetUrl( $file )
	{
		return preg_replace( '/\s/', '%20', plugins_url( $this->path( 'ASSETS_DIR_NAME', $file ), __FILE__ ) );
	}

	/**
	 * Get absolute url for Instaberg asset file.
	 *
	 * Assets are css, javascript, less files and images.
	 *
	 * @since 4.2
	 *
	 * @param $file
	 *
	 * @return string
	 */
	public function templatesUrl( $file )
	{
		return preg_replace( '/\s/', '%20', plugins_url( $this->path( 'TEMPLATES_DIR_NAME', $file ), __FILE__ ) );
	}

	public function print_filters_for( $hook = '' )
	{
		global $wp_filter;
		if( empty( $hook ) || !isset( $wp_filter[$hook] ) )
			return;
	}


	public function admin_load_styles()
	{
		// admin notices - css styles
		wp_enqueue_style( 'Instaberg-admin-notices-style', Instaberg_asset_url( 'admin_notices.css' ), array(), Instaberg_VERSION );
	}

	public function admin_load_scripts() {
		wp_enqueue_media();
	}

	public function prepareForInList($v) {
		return "'".$v."'";
	}

	public function template_path()
	{
		 return apply_filters( 'Instaberg_template_path', 'templates/' );
	}

	public function ajax_register()
	{
		parse_str( $_REQUEST['params'], $params );

		$ipc = isset($params['Instaberg_iw-validation-token']) ? $params['Instaberg_iw-validation-token'] : '';
		$email = isset($params['Instaberg_iw-validation-email']) ? $params['Instaberg_iw-validation-email'] : '';

		$link = "http://cc.aa-team.com/validation/validate.php?ipc=%s&email=%s&app=" . $this->alias;
		$aa_server_response = wp_remote_retrieve_body( wp_remote_get( esc_url( sprintf( $link, $ipc,$email, $email ) ) ) );
		if( $aa_server_response ){
			$aa_server_response = json_decode( $aa_server_response, true );
			if( $aa_server_response ){

				if( $aa_server_response['status'] == 'valid' ){
					if( isset($aa_server_response['html']) ){
						update_option( "_" . $this->alias . "_register_html", $aa_server_response['html'] );

						die( json_encode( array( 'status' => 'valid' ) ));
					}
				}
			}
		}
	}

	public function getAllGfonts( $what='all' )
	{
		$file = $this->path( 'ASSETS_DIR', 'fonts/google-webfonts.json' );
		$fonts = json_decode( $this->wp_filesystem->get_contents( $file ), true);

		$ret_fonts = array();
		$ret_fonts['system'] = 'Arial';
		$ret_fonts['system'] = 'Arial Black';
		$ret_fonts['system'] = 'Impact';
		$ret_fonts['system'] = 'Verdana';
		if(count($fonts) > 0 ){
			foreach ( $fonts as $font ) {
				$ret_fonts[$font] = $font;
			}
		}

		if( $what == 'all' ){
			return $ret_fonts;
		}
	}

	public function ajax_request() {
		$action = isset($_REQUEST['sub_action']) ? $_REQUEST['sub_action'] : 'none';

		$actions_allowed = array( 'search_wp_post' );

		if ( ! in_array($action, $actions_allowed) ) {
			$this->print_response( 'invalid', 'invalid action (not allowed)!' );
		}

		if ( $action == 'search_wp_post' ) {
			$posts = $this->search_wp_post( array(
				'post_type' 		=> 'post',
				'post_status'		=> 'publish',
				'posts_per_page'   	=> -1,
				's'					=> $_REQUEST['keyword']

			) );

			if( $posts && count($posts) ){
				$this->print_response( 'valid', 'data ok', $posts );
			}else{
				$this->print_response( 'invalid', 'Unable to get any post for your searched keyword!' );
			}
		}

		die("invalid action!");
	}

	private function print_response( $status='valid', $msg='', $data=array() ) {
		die( json_encode( array(
			'status' => $status,
			'msg' => $msg,
			'data' => $data
		) ) );
	}

	public function notification_bar_frontend()
	{
		require_once Instaberg()->path( 'TEMPLATES_DIR', 'notification-bar.php' );
	}

	public function default_colors()
	{
		$color = array();
		$color['Black'] = explode(",", '0,0,0' );
		$color['White'] = explode(",", '255,255,255' );
		$color['Red'] = explode(",", '244,67,54' );
		$color['Pink'] = explode(",", '233,30,99' );
		$color['Purple'] = explode(",", '156,39,176' );
		$color['Deep Purple'] = explode(",", '103,58,183' );
		$color['Indigo'] = explode(",", '63,81,181' );
		$color['Blue'] = explode(",", '33,150,243' );
		$color['Light Blue'] = explode(",", '3,169,244' );
		$color['Cyan'] = explode(",", '0,188,212' );
		$color['Teal'] = explode(",", '0,150,136' );
		$color['Green'] = explode(",", '76,175,80' );
		$color['Light Green'] = explode(",", '139,195,74' );
		$color['Lime'] = explode(",", '205,220,57' );
		$color['Yellow'] = explode(",", '255,235,59' );
		$color['Amber'] = explode(",", '255,193,7' );
		$color['Orange'] = explode(",", '255,152,0' );
		$color['Deep Orange'] = explode(",", '255,87,34' );
		$color['Brown'] = explode(",", '121,85,72' );
		$color['Grey'] = explode(",", '158,158,158' );
		$color['Blue Grey'] = explode(",", '96,125,139' );
		return $color;
	}

	public function the_colors()
	{
		if( isset($this->settings['settings']['the_colors']) ){
			$color = array();
			if( count($this->settings['settings']['the_colors']) > 0 ){
				foreach ($this->settings['settings']['the_colors']['name'] as $key => $name) {
					$hex_color = $this->settings['settings']['the_colors']['color'][$key];
					$color[$name] = $this->hex2rgb( $hex_color );
				}
			}

			if( count($color) > 0 ){
				return $color;
			}
		}
		return $this->default_colors();
	}

	public function isValidColorName( $colorName='' )
	{
		if( $this->the_colors() ){
			if( in_array( $colorName, array_keys( $this->the_colors() ) ) ) {
				$colors = $this->the_colors();
				return $colors[$colorName];
			}
		}

		return false;
	}

	public function rgb2html($r, $g=-1, $b=-1)
	{
		if (is_array($r) && sizeof($r) == 3)
			list($r, $g, $b) = $r;

		$r = intval($r); $g = intval($g);
		$b = intval($b);

		$r = dechex($r<0?0:($r>255?255:$r));
		$g = dechex($g<0?0:($g>255?255:$g));
		$b = dechex($b<0?0:($b>255?255:$b));

		$color = (strlen($r) < 2?'0':'').$r;
		$color .= (strlen($g) < 2?'0':'').$g;
		$color .= (strlen($b) < 2?'0':'').$b;
		return '#'.$color;
	}

	function hex2rgb( $hex, $alpha = false )
	{
		$hex      = str_replace('#', '', $hex);
		$length   = strlen($hex);
		$rgb[0] = hexdec($length == 6 ? substr($hex, 0, 2) : ($length == 3 ? str_repeat(substr($hex, 0, 1), 2) : 0));
		$rgb[1] = hexdec($length == 6 ? substr($hex, 2, 2) : ($length == 3 ? str_repeat(substr($hex, 1, 1), 2) : 0));
		$rgb[2] = hexdec($length == 6 ? substr($hex, 4, 2) : ($length == 3 ? str_repeat(substr($hex, 2, 1), 2) : 0));
		if ( $alpha ) {
			$rgb['a'] = $alpha;
		}
		return $rgb;
	}

	public function range( $start, $end, $step=1 )
	{
		$values = array();
		foreach( range( $start, $end, $step ) as $value ){
			$values[$value] = $value;
		}

		return $values;
	}

	private function search_wp_post( $args=array() )
	{
		$query = new WP_Query( $args );
		$base = array();

		if( $query->have_posts() ){
			foreach( $query->posts as $post ){

				$excerpt = get_the_excerpt( $post );
				if( $excerpt == "" ){
					$excerpt = $this->get_first_paragraph( $post->post_content );
				}

				$base[] = array(
					'ID' => $post->ID,
					'post_title' => $post->post_title,
					'excerpt' => $excerpt
				);
			}
		}

		return $base;
	}

	private function get_first_paragraph( $post_content='' )
	{
		$str = wpautop( do_shortcode( $post_content ) );
		$str = substr( $str, 0, strpos( $str, '</p>' ) + 4 );
		$str = strip_tags($str, '<a><strong><em>');
		return '<p>' . $str . '</p>';
	}
}

/**
 * Main Instaberg manager.
 * @var Instaberg $Instaberg - instance of composer management.
 * @since 1.0
 */
global $Instaberg;
$Instaberg = new Instaberg();
